* ---------------------------------------------------------------- SaMaxMin
      Program SaMaxMin

* Reads a pair of SMC filenames from a control file, loops over a series of rotation 
* angles, for each forming the time series for the rotation angle and computing the Sa.  
* The max and min over the set of rotation angles is found and printed.
  
* Structure of control file:
* Line 1 follows:
*!Control file for program SaMaxMin
*!Summary file name:
* SaMaxMin.sum
*!dazm, azm_low, azm_high
*  1.0 0.0 179.0
*!nperiods followed by the specific periods:
* 13
* 0.1
* 0.15
* 0.2
* 0.3
* 0.4
* 0.5
* 0.75
* 1
* 1.5
* 2
* 3
* 4
* 5
*!Damping value (damp):
* 0.05
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -10.0  
*! Skip analysis (including rotation) if npts differ by more than:
*  2
*! Extension for output file (append to end of first file name)
* .samaxmin.asc
*! list of file names (assumes pairs of files) 
*    0516a.smc
*    0516c.smc
*    0526a.smc
*    0526c.smc
*  StoP



* Sign conventions:  the azimuth of the radial component is 180+azimuth from
* station to reference point, and the transverse component is this radial 
* azimuth plus 90 degrees.  This should be equivalent to the convention
* in Aki and Richards, Fig. 4.10, p. 96.

* This is incomplete, because the program will eventually also deal with case
* for which i_spcfy_fname = 1.  A program that works 
* with this is smcrtold (the old version of smc_rot)

* Dates: 10/07/06 - Written by D. Boore, based on grot50.  
 
               
  
      character char_head(11,2)*80, comments(200,2)*80
      character cmnts2skip(20)*80, path_in*40, path_out*40      
       
      integer idate_time_values(8)
      
      real time_sec_start, time_sec_stop

      integer int_head(48,2), nc_f_in(2),  nc_f_out, nc_rc(2)
      integer npts(2)

      real real_head(50,2) 
      real alat_sta(2), along_sta(2)
      real a_in(:,:), a_rot(:) 
      allocatable :: a_in, a_rot
       
      real period(120), rd, psa, 
     :    samax(120), samin(120), rot_max(120), rot_min(120)  
      real omega, omega2
          

      real rnull(2), sps(2), azm_in(2), rot_angle(400)
      character buf_c*80, buf4_c*4, f_ctl*80, f_sum*80,
     :          buf_out*79
      character  f_in(2)*80, f_out*100, f_ext*80, f_penalty*100,
     :           c_az*7, rc(2)*12
      character per4az_c*5, colhead1(10)*12, colhead2(10)*12, frmt_c*80
      character use_nga_per4gm*10
      logical f_ctl_exist
      logical rmv_trnd
      
      pi = 4.0*atan(1.0)
      twopi = 2.0*pi
      d2r = pi/180.0
      nper_dimen = 120
      nazm_dimen = 400
      
      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = samaxmin.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'samaxmin.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      call trim_c(f_sum,nc_f_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')    
      write(nu_sum,'(a)') ' Summary file for program GMeanRot'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl
      do i = 1, nc_comments
        write(nu_sum,'(a)') cmnts2skip(i)
      end do
      write(nu_sum,'(a)') f_sum

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) dazm, azm_low, azm_high
      write(nu_sum,*) dazm, azm_low, azm_high
      nazm = (azm_high - dazm - azm_low)/dazm + 1.3
      if (nazm .gt. nazm_dimen) then
        nazm = nazm_dimen
        write(nu_sum,'(a,1x,i3)') ' resetting nazm to ', nazm
        dazm = (azm_high - dazm - azm_low)/float(nazm - 1)
        write(nu_sum,'(a,1x,i3)') ' and dazm to ', dazm
      else        
        write(nu_sum,'(a,1x,i3)') ' nazm = ', nazm
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nper
      write(nu_sum,*) nper
      read(nu_ctl,*) (period(i),i=1,nper)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) damp
      write(nu_sum,*) damp
      
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_sum,*) t4mean_in
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) npts_skip
      write(nu_sum,*) npts_skip
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      f_ext = ' '
      read(nu_ctl,'(a)') f_ext
      call trim_c(f_ext,nc_f_ext)
      write(nu_sum,'(a)') f_ext(1:nc_f_ext)
         
 

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      write(nu_sum, '(a)') ' Just before 1000 continue '

1000  continue ! Loop over files

        write(nu_sum,*)

 
* First get max dimension and allocate arrays:
        do j = 1, 2
          buf_c = ' '
          read(nu_ctl,'(a)') buf_c
          call trim_c(buf_c,nc_buf)
          buf4_c = ' '
          buf4_c = buf_c(1:4)
          call upstr(buf4_c)

          if (buf4_c(1:4) .eq. 'STOP') then
            write(*, '(a)') 
     :           ' buf_c = '//buf_c(1:nc_buf)//'; QUITTING'
            close(unit=nu_sum)
            close(nu_ctl)
            stop
          end if

          call date_and_time(values=idate_time_values)
          time_sec_start = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                     float(idate_time_values(8))/1000.0
     
          f_in(j) = ' '
          f_in(j) = buf_c(1:nc_buf)
          call trim_c(f_in(j),nc_f_in(j))
        
          write(nu_sum,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          write(*,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          call SMC_NPTS(f_in(j)(1:nc_f_in(j)), npts(j), sps(j))
        end do
        
        write(nu_sum,'(1x,a,1x,i3)') ' nper  = ', nper 

        write(nu_sum,'(a,1x,i3, 1x,f8.4)') 
     :          ' nper, period(nper) = ',
     :            nper, period(nper)
       
        max_dimen = max0( npts(1), npts(2) )
        allocate( a_in(max_dimen,2), a_rot(max_dimen) )
 
* Second, loop over input files, reading data:
        do j = 1, 2
          call SMCRead(f_in(j)(1:nc_f_in(j)), 0.0, 100000.0,
     :                 a_in(1,j), char_head(1,j), 
     :                 int_head(1,j), real_head(1,j), 
     :                 comments(1,j))

          npts(j)  = int_head(17,j)  ! should check equality of both time series
          rnull(j) = real_head(1,j)
          azm_in(j) = real(int_head(14,j))
        end do

* Check for equality of sps:
        if ( sps(1) .ne. sps(2)) then
            write(nu_sum,'(a,1p, 2(1x,e12.5))') 
     :      ' SKIPPING: sps1 .ne. sps2; sps1, sps2 = ', 
     :        sps(1), sps(2)
            go to 1000
        end if
* Check for equality of npts:
        if ( iabs(npts(1) - npts(2)) .gt. npts_skip) then
            write(nu_sum,'(a,3(1x,i6))') 
     :      ' SKIPPING: iabs(npts1 - npts2) > '//
     :      'npts_skip; npts1, npts2, npts_skip = ', 
     :        npts(1), npts(2), npts_skip
            go to 1000
        end if
* Check for equality of rnull:
        if ( rnull(1) .ne. rnull(2)) then
          write(nu_sum,'(a,1p2(1x,e12.5))') 
     :      ' SKIPPING: rnull1 .ne. rnull2; rnull1, rnull2 = ', 
     :        rnull(1), rnull(2)
            go to 1000
        end if
* Check for azimuths being 90 degrees apart (compute the dot product):
        dot = cos(d2r*azm_in(1))*cos(d2r*azm_in(2))+
     :        sin(d2r*azm_in(1))*sin(d2r*azm_in(2))
        eps = 0.001
        if (abs(dot) .gt. eps) then
          write(*,'(a)') '  Dot product not 0.0; see SUM file'//
     :                   ' for details; SKIPPING!!!'
          write(nu_sum,'(a)') '  azm_in(1,2), dot = '
          write(nu_sum,*) azm_in(1), azm_in(2), dot
            go to 1000
        end if

         
* Passed all equality tests.

      
* Step: Remove a mean from the whole record

*  Determine the appropriate mean:
        do j = 1, 2
          t4mean = t4mean_in
          if(t4mean .lt. 0.0) then  ! do not remove a mean
            avg = 0.0
          else
            tend = float(npts(j)-1)/sps(j)
            if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
              t4mean = tend  ! use whole record
            end if
          end if
          n_t4mean = sps(j) * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a_in(i,j))
          end do
          avg = cum_avg/float(n_t4mean)
*          avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
          do i = 1, npts(j)
            a_in(i,j) = a_in(i,j) - avg
          end do
        end do
        
*  Finished reading and massaging the two input time series  
 
      do iper = 1, nper_dimen
        samax(iper) = 0.0
        samin(iper) = 10000.0
        rot_max(iper) = 0.0
        rot_min(iper) = 0.0
      end do

* Step: Loop over azimuths:

      do iz = 1, nazm
        
        rot_angle(iz) = float(iz-1)*dazm + azm_low
            
            
* Rotate the time series into a single time series  

        do i = 1, npts(1)
          cs = cos(d2r*rot_angle(iz))
          sn = sin(d2r*rot_angle(iz))
          a_rot(i) = a_in(i,1)*cs + a_in(i,2)*sn
        end do             


* Step: Loop over periods:

        do iper = 1, nper
        
* Compute response spectra for this period:
          omega = twopi/period(iper)
          omega2 = omega**2
          
          dt = 1.0/sps(1)
          call rdcalcdp(a_rot, npts(1), omega,
     :        damp, dt, rd)
          psa = omega2 * rd/981.0 
          
          if (psa .lt. samin(iper)) then
            samin(iper) = psa
            rot_min(iper) = rot_angle(iz)
          end if
          if (psa .gt. samax(iper)) then
            samax(iper) = psa
            rot_max(iper) = rot_angle(iz)
          end if
          
        end do
 
      end do  ! end loop over azimuths.
 
       
 

* Now write out the samax, thetamax, min values:

        f_out = ' '
        f_out = f_in(1)(1:nc_f_in(1))//f_ext(1:nc_f_ext)
        call trim_c(f_out,nc_f_out)
        call get_lun(nu_out)
        open(nu_out,file=f_out(1:nc_f_out),status='unknown')
        write(nu_sum,'(a)') ' Output file = '//f_out(1:nc_f_out)
        
        write(nu_out,'(6x,a, 
     :                 6x,a,1x,a, 1x,a,
     :                 6x,a,1x,a, 1x,a)') 
     :    'per', 
     :    'SaMax', 'RotMax',  'ThetaMax',
     :    'SaMin', 'RotMin',  'ThetaMin'

        do iper = 1, nper
        
          theta_max = rot_max(iper)+ azm_in(1)
          if (theta_max .gt. 360.0) then
            theta_max = theta_max -360.0
          else if (theta_max .gt. 180.0) then
            theta_max = theta_max -180.0
          end if
          
          theta_min = rot_min(iper)+ azm_in(1)
          if (theta_min .gt. 360.0) then
            theta_min = theta_min -360.0
          else if (theta_min .gt. 180.0) then
            theta_min = theta_min -180.0
          end if
          
        
          write(nu_out,'(1x,f8.4,2(1p,1x,e10.3,0p,1x,f6.1,3x,f6.1))') 
     :            period(iper), 
     :            samax(iper), rot_max(iper), theta_max,
     :            samin(iper), rot_min(iper), theta_min
     
        end do
        
        close(nu_out)
        
     
        
        deallocate( a_in )
        deallocate( a_rot )
         
         
      
        call date_and_time(values=idate_time_values)
        time_sec_stop = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                    float(idate_time_values(8))/1000.0
     
       write(nu_sum,'(a,1x,f7.3)') ' calc time = ',
     :                     time_sec_stop - time_sec_start
     
      go to 1000    ! loop back for another pair of components

      end
* ---------------------------------------------------------------- SMC_Rot

      include '\forprogs\acc2vd.for'     
      include '\forprogs\Trapezoidal_Integration.for'
      include '\forprogs\rmvtrend.for'     
      include '\forprogs\rotate.for'     
      include '\forprogs\get_lun.for'     
      include '\forprogs\skip.for'     
      include '\forprogs\smcread.for'     
      include '\forprogs\smcwrite.for'     
      include '\forprogs\mnmaxidx.for'     
      include '\forprogs\abs_mnmaxidx.for'     
      include '\forprogs\trim_c.for'     
      include '\forprogs\upstr.for'     
      include '\forprogs\smc_npts.for'     
      include '\forprogs\distaz.for'     
      include '\forprogs\rc_subs.for'     
      include '\forprogs\skipcmnt.for'     
      include '\forprogs\rdcalcdp.for'
      include '\forprogs\moment.for'
      include '\forprogs\select.for'
  

*      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)

 
*Write a file with az1, daz, (period, gmean, gmean-%, gmean_50%, gmean+%), repeated for a set of periods specficied in input  


      
