! ------------------------------------------------------------------- SMC2CAV
      Program SMC2CAV


! Computes CAV for smc file

! Input from a control file; here is an example:
!!Control file for program SMC2CAV
!! amin4cav (in g, 0.025 for standardized CAV, 0.0 for regular CAV)
! 0.0
!! 0.025
!!Name of output file 
! smc2cav.out
!!List of files 
! t0781pea.smc
! t0781ped.smc
!Stop
 
! Dates: 11/20/08 - Written by D.M. Boore, patterned after SMC2JERK
!        08/26/10 - Use modified version of cav_compute (pass units flag
!                   through argument list).
!        06/25/19 - Increase precision of output
!        06/27/19 - Add amin4cav to argument list of cav_compute, and to control file. 
!
! Dimension and declaration statements:

      character precision*8

 
      real, allocatable :: a(:) 

      integer npts, nu_out, status
      real tend
      real*8 cum_avg

      character date_c*8, time_c*11, ext4out*20

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*120, buf4*4, 
     :          f_a*120, f_out*1200

      character f_ctl*40
      logical f_ctl_exist, f_a_exist, rmv_trnd, use_v0_d0

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2CAV.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2cav.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) amin4cav
  
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

      write(nu_out,'(a)') ' Output of program SMC2CAV:'
      write(nu_out,'(2a)') '   Control file = ', f_ctl
      write(nu_out,'(a, 1x, es10.3)') ' amin4cav = ', amin4cav

      write(nu_out,'(78x,a, 3x,a, 4x,a, 6x,a, 4x,a)') 
     :                  'f_a', 'npts', 'sps', 'pga(g)', 'cav(g-s)'
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

       
      loop over files: DO
      buf = ' '
      read(nu_ctl,'(a)',iostat=status) buf
      IF (status /= 0) THEN
        EXIT loop over files
      END IF
      call trim_c(buf,nc_buf)
      buf4 = ' '
      buf4 = buf(1:4)
      call upstr(buf4)
      IF(nc_buf == 0 .or. buf(1:1) == '!') CYCLE loop over files
      IF(buf4(1:4) == 'STOP') THEN
        EXIT loop over files
      END IF
       
      f_a = ' '
      f_a = buf(1:nc_buf)
      nc_f_a = nc_buf     
      inquire(file=f_a(1:nc_f_a), exist=f_a_exist)
 
      if (.not. f_a_exist) then
          write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE loop over files
      end if

! Passed tests; proceed

 
      call smc_npts(f_a(1:nc_f_a), npts, sps)

 
      allocate( a(npts) )

! Now read the data from the files:

      write(*,'(1x,a)') 
     :   'Processing file:', f_a(1:nc_f_a)

      tskip = 0.0
      tlength = -1.0

      call SMCRead(f_a(1:nc_f_a), tskip, tlength,
     :               a,  
     :               char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      rnull = real_head(1)
 
      sps = real_head(2)
      ncomments_a = int_head(16)
      npts = int_head(17)
      dt = 1.0/sps
      tend = float(npts-1)*dt

! Get peak motions:
      
      call mnmaxidx(a, 1, npts, 1,
     :             amin, amax, indx_min, indx_max) 
      peak_mtn = amax
     
      if (abs(amin) .gt. abs(amax)) peak_mtn = amin

      pga_g = abs(peak_mtn)/981.0
 
! Compute cav

!       where iunits is:
!         0 = g
!         1 = cm/s/s
!         2 = m/s/s

      iunits = 1

      call cav_compute(a, amin4cav, iunits, npts, sps, cav)
      deallocate (a)

     
! Write output:

      write(nu_out,'(1x,a80, 1x,i6, 1x,f6.1, 1x,es11.4, 1x,es11.4)')  
     :    f_a(1:nc_f_a), npts, sps, pga_g, cav
     
      END DO loop over files

 
! Close control and output files:

      close(nu_ctl)
      close(nu_out)

      stop
      end
! ------------------------------------------------------------------- SMC2CAV


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\datetime.for'
      include '\forprogs\cav_compute.for'
 