! ------------------------------------------------------------------- SMC2Husid
      Program SMC2Husid


! Produces a file of the integral square acceleration from acceleration file

! The output file name has the same stem name as the input file, with ".h.asc"
! added to the file name.  The output file takes advantage of the longer
! file names in F90, and the file is written as a column file with columns
! containing time, a_sq_int, a_sq_int_norm, a_sq_int_arias.

! Input from a control file; here is an example:
!!Control file for program SMC2Husid
!! As many comment lines as desired, each starting with "!"
!! Revision of program involving a change in the control file on this date:
!   12/08/13
!!Name of summary file:
! smc2husid.sum
!!mean info:
!! type of mean (0=no correction, 1=standard average; -1=specify mean)
!! t4mean_strt, t4mean_stop, mean (use if type = -1)
!! NOTE: need entries even if the parameter is not used (because read
!! info use list-directed [unformatted] input) 
!! Note that if working with a processed record, it is
!! inadvisible to remove the mean as this may produce unintended
!! distortions at long periods --- so in most cases set t4mean < 0.0):
! 0  1.0 20.0  -999.0
!!fractile_low, fractile_high, delta_fractile
! 0.05 0.95 0.05
!!fractile_durstart
! 0.05
!!fractile_ref
! 0.50
!!Stem name of output file containing the durations (".out" will be added):
! smc2husid_durations
!!Write ascii files with husid time series (Y/N)?
! Y
!!List of files 
! t0781pea.smc
! t0781ped.smc
!Stop

! Dates: 07/28/03 - Written by D.M. Boore, patterned after SMC2VD
!        10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!        02/06/09 - Correct error in t4mean_strt and t4mean_stop being reset from their input
!                   values when processing a list of files.  The problem is that if a file with
!                   a shorter record length is processed before one with a longer record
!                   length, a mean supposedly determined for the whole record was determined
!                   only for the length of the shorter preceeding record.
!        12/31/10 - Add acceleration time series to output (both as is and normalized to unity).
!                 - Changed name to smc2husid
!        05/20/11 - Correct error--t4mean_strt, t4mean_stop not defined
!        06/03/13 - Compute various duration measures and write to a summary file.
!        06/04/13 - Add a change date to control file.
!                   Allow not writing of individual files, and ask for name of duration file
!        07/30/13 - Allow more flexibility in the durations to be written to the output file,
!                   and write the ratios of the durations to a reference duration.  More precisely,
!                   this will be this ratio: [dur(i)-dur(1)]/[dur(iref)-dur(1)]. In other words,
!                   duration is defined relative to the time corresponding to the first fractile.
!        09/01/14 - Increase length of input file name
        


c
c Dimension and declaration statements:

      real a(:), a_sq(:), a_sq_int(:), a_sq_int_norm(:), time(:)
      allocatable :: a, a_sq, a_sq_int, a_sq_int_norm, time
      
      real a_sq_int_trap(:), a_sq_int_trap_norm(:) 
      allocatable :: a_sq_int_trap, a_sq_int_trap_norm
      
      real fractile(100), duration(100), 
     :     t4fractile(100), duration_norm(100)
     
      real                duration_trap(100), 
     :   t4fractile_trap(100), duration_trap_norm(100)
      
      integer status

      integer npts, nc_f_a, nc_f_out
      real tend
      real*8 cum_avg

      character date_c*8, time_c*11

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)
      
      character colhead(200)*12

      character version_ctl*8, version_in*30

      character buf*200, buf_upper*4, 
     :        f_sum*200, f_a*200, f_out*200, 
     :        f_stem*200, f_durations*200, f_durations_trap*200

      character f_ctl*200
      logical f_ctl_exist, f_a_exist, rmv_trnd, write_individual_files

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2Husid.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2Husid.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')


!Check version of control file
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      version_in = ' '
      read(nu_ctl,'(a)') version_in
      call trim_c(version_in,nc_version_in)
      
      version_ctl = ' '
      version_ctl = '12/08/13'
      call trim_c(version_ctl,nc_version_ctl)

      IF (version_ctl(1:nc_version_ctl) /= 
     :    version_in(1:nc_version_in)) THEN
        print *,' The control file has the wrong version date.'
        print *,' The proper date is '//version_ctl(1:nc_version_ctl)
        print *,' Program is STOPPING!'
        close(nu_ctl)
        stop
      END IF
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2Husid:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :     itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_sum,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_sum,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :     fractile_low, fractile_high, delta_fractile

      nfractiles = 
     :   int((fractile_high-fractile_low)/delta_fractile+0.1)+1
      write(nu_sum,'(a)') 
     :  ' fractile_low, fractile_high, delta_fractile, nfractiles = '
      write(nu_sum,*) 
     :    fractile_low, fractile_high, delta_fractile, nfractiles
      do i = 1, nfractiles
        fractile(i) = fractile_low + delta_fractile*real(i-1)
      end do

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :     fractile_durstart
      write(nu_sum,'(a)') 
     :  ' fractile_durstart = '
      write(nu_sum,*) 
     :    fractile_durstart    

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :     fractile_ref
      write(nu_sum,'(a)') 
     :  ' fractile_ref = '
      write(nu_sum,*) 
     :    fractile_ref
     
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      f_stem = ' '
      read(nu_ctl,'(a)') f_stem
      call trim_c(f_stem, nc_f_stem)

      f_durations = f_stem(1:nc_f_stem)//'.out'
      call trim_c(f_durations, nc_f_durations)
      call get_lun(nu_dur)
      open(unit=nu_dur,file=f_durations(1:nc_f_durations),
     :    status='unknown')
      write(nu_sum,'(a, 1x, i3, 1x, a)') 
     :   ' nu_dur, dur file: ', 
     :     nu_dur, f_durations(1:nc_f_durations)
 
      f_durations_trap = f_stem(1:nc_f_stem)//'.trap.out'
      call trim_c(f_durations_trap, nc_f_durations_trap)
      call get_lun(nu_dur_trap)
      open(unit=nu_dur_trap,
     :    file=f_durations_trap(1:nc_f_durations_trap),
     :    status='unknown')
      write(nu_sum,'(a, 1x, i3, 1x, a)') 
     :   ' nu_dur_trap, dur_trap file: ', 
     :     nu_dur_trap, f_durations_trap(1:nc_f_durations_trap)


      
      colhead = ' '
      do i = 1, nfractiles
!                     123456789012
        colhead(i) = '       t0000'
        write(colhead(i)(9:12),'(f4.2)') fractile(i)
!                                123456789012
        colhead(i+nfractiles) = '       d0000'
        write(colhead(i+nfractiles)(9:12),'(f4.2)') fractile(i)
!                                  123456789012
        colhead(i+2*nfractiles) = '   dnorm0000'
        write(colhead(i+2*nfractiles)(9:12),'(f4.2)') fractile(i)
      end do
 
      write(nu_dur,'(
     :  62x,a, 1x,a, 
     :  200(1x,a12)
     :                            )') 
     :  'file_name', 'index',
     :  (colhead(i), i = 1, 3*nfractiles)
     
      write(nu_dur_trap,'(
     :  62x,a, 1x,a, 
     :  200(1x,a12)
     :                            )') 
     :  'file_name', 'index',
     :  (colhead(i), i = 1, 3*nfractiles)
     
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
        write_individual_files = .true.
      else
        write_individual_files = .false.
      end if
      
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      num = 0
      
      loop over files: DO
        buf = ' '
        read(nu_ctl,'(a)',iostat=status) buf
        if (status /= 0) then
          EXIT loop over files
        end if
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') CYCLE
        if(buf_upper(1:4) == 'STOP') EXIT
      
        f_a = ' '
        f_a = buf(1:nc_buf)
        nc_f_a = nc_buf     
        inquire(file=f_a(1:nc_f_a), exist=f_a_exist)
        if (.not. f_a_exist) then
            write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
            CYCLE
        end if

! Passed tests; proceed

        num = num + 1

! Get number of points on this loop:

        call smc_npts(f_a(1:nc_f_a), npts, sps)

!      write(nu_sum,*)
!      write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

        allocate( a(npts+5), a_sq(npts+1),
     :            a_sq_int(npts+5), a_sq_int_norm(npts+5), 
     :            time(npts+5),
     :            a_sq_int_trap(npts+5), a_sq_int_trap_norm(npts+5))

! Now read the data from the files:

        write(*,'(1x,a)') 
     :   'Processing file:', f_a(1:nc_f_a)

        tskip = 0.0
        tlength = -1.0

        call SMCRead(f_a(1:nc_f_a), tskip, tlength,
     :               a,  
     :               char_head, int_head, real_head, comments)

        rnull = real_head(1)
        v0 = 0.0
        d0 = 0.0
  
        sps = real_head(2)
        ncomments_a = int_head(16)
        npts = int_head(17)
        dt = 1.0/sps
        tend = float(npts-1)*dt
        
        do i = 1, npts
          time(i) = real(i-1)*dt
        end do



! Step 1: Remove a mean from the whole record

        n_t4mean_strt_in = int(t4mean_strt_in*sps+0.001) + 1
        n_t4mean_stop_in = int(t4mean_stop_in*sps+0.001) + 1
      
        n_t4mean_strt = n_t4mean_strt_in
        n_t4mean_stop = n_t4mean_stop_in

        t4mean_strt = t4mean_strt_in
        t4mean_stop = t4mean_stop_in
        
!  Determine the appropriate mean:
      if (itype_mean == 0) then
        n_t4mean = 0
        avg = 0.0
      else if (itype_mean == -1) then   ! use specified mean
        avg = spec_mean
      else if (itype_mean == 1) then
        if (t4mean_strt .lt. 0.0) t4mean_strt = 0.0
        if (t4mean_stop .gt. tend) t4mean_stop = tend
        n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
        n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
        n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
        t4mean = float(n_t4mean -1)/sps
!        write(nu_sum,'(a,1x,f6.2)') ' t4mean = ', t4mean
        cum_avg = 0.0
        do i = n_t4mean_strt, n_t4mean_stop
          cum_avg = cum_avg + dble(a(i))
        end do
        avg = sngl(cum_avg/n_t4mean)
      else            ! shouldn't be here!
        write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        write(*) '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        deallocate (a, a_sq_int, a_sq_int_trap)
        EXIT loop over files
      end if

!  Remove the mean from the whole record:
      do i = 1, npts
        a(i) = a(i) - avg
        a_sq(i) = a(i)**2
      end do
      call mnmaxidx(a_sq,1,npts,1,
     :                    a_sq_min,a_sq_max,indx_min,indx_max) 

! Step2: Integrate the squared acceleration

      rmv_trnd = .false.
      call accsqint(a, npts, dt, rmv_trnd, a_sq_int)
      call accsqint_trapezoidal(a, npts, dt, rmv_trnd, a_sq_int_trap)

! Step 3: Write output:

      if (write_individual_files) then 
        indp = index(f_a,'.')
        f_out = ' '
        f_out = f_a(1:nc_f_a)//'.h.asc'
        call trim_c(f_out,nc_f_out)
        call get_lun(nu_out)
        open(unit=nu_out,file=f_out,status='unknown')


        write(*,'(1x,a)') 
     :   'Writing acc_sq_int output to file:', f_out(1:nc_f_out)

        iflag = 0 ! not used by current PC version of datetime; Chris
                  ! Stephens would modify datetime.for to include Unix routines
        call datetime(date_c, time_c, iflag)
        
        ncomments = ncomments_a
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| Int of Squared Acceleration, from Processing acc '//
     :      'file '//f_a(1:nc_f_a)//' using program SMC2Husid'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '|   on '//date_c//' at time '//time_c

        if (itype_mean .eq. 0) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
        else if (itype_mean .eq. 1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
        else if (itype_mean .eq. -1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
        else
          write(*,'(a)') ' Not supposed to be here.'
        end if

        ncomments = ncomments + 1
        comments(ncomments) = '|   resulting in output file '//
     :      f_out(1:nc_f_out)

        ncomments = ncomments + 1
        comments(ncomments) = '|'
      end if

      call mnmaxidx(a,1,npts,1,
     :                    amin,amax,indx_min,indx_max) 
      
      pga = abs(amin)
      if (pga .lt. abs(amax)) pga = abs(amax)

! Try to guess units of accel, so can define g properly:
      if (pga .gt. 10.0) then
        aunits = 1.0  ! assume cm/s/s, do not change units of a
      else
        aunits = 981.0
      end if      

      anorm = a_sq_int(npts)
      anorm_trap = a_sq_int_trap(npts)
      pi = 4.0*atan(1.0)
      ai = (pi*(anorm*aunits**2))/(2.0*981.0) 

      do i = 1, npts
        a_sq_int_norm(i) = a_sq_int(i)/anorm
        a_sq_int_trap_norm(i) = a_sq_int_trap(i)/anorm_trap
      end do

      if (write_individual_files) then 
        write(nu_out, '(a,1x,i4)') 
     :   ' nheaders before column labels = ', ncomments + 1
        do i = 1, ncomments
          write(nu_out,'(a)') comments(i)
        end do
        write(nu_out, '(6x,a,
     :   14x,a, 9x,a, 
     :   7x,a, 5x,a, 10x,a,
     :   2x,a, 1x,a, 5x,a,
     :   11x,a, 6x,a
     :                             )')
     :  'time', 
     :   'a', 'a_norm', 
     :   'a_sq_int', 'a_sq_int_nrm', 'arias',
     :   'a_sq_int_trap', 'a_sq_int_trap_nrm', 'arias_trap',
     :   'a_sq', 'a_sq_norm'
        do i = 1, npts
          write(nu_out,'(1x,f9.4,
     :     1x,es14.5, 1x,es14.5, 
     :     1x,es14.5, 3x,es14.5, 1x,es14.5,
     :     1x,es14.5, 4x,es14.5, 1x,es14.5,
     :     1x,es14.5, 1x,es14.5
     :                                        )')
     :     float(i-1)*dt, 
     :     a(i), a(i)/pga, 
     :     a_sq_int(i), a_sq_int_norm(i), ai*a_sq_int_norm(i),
     :     a_sq_int_trap(i), a_sq_int_trap_norm(i),
     :                    ai*a_sq_int_trap_norm(i),
     :     a_sq(i), a_sq(i)/a_sq_max
        end do
        close(nu_out) 
      end if

!Find durations:
      ynull =-999.0
      do i = 1, nfractiles
         call interpolate_1d(a_sq_int_norm, time, npts, ynull,
     :                          fractile(i), t4fractile(i))
      end do
        
      call interpolate_1d(a_sq_int_norm, time, npts, ynull,
     :                   fractile_durstart, t4fractile_durstart)
      call interpolate_1d(a_sq_int_norm, time, npts, ynull,
     :                   fractile_ref, t4fractile_ref)
      duration_ref = t4fractile_ref - t4fractile_durstart     
      do i = 1, nfractiles
        duration(i) =  t4fractile(i) -  t4fractile_durstart   
        duration_norm(i) =  duration(i)/duration_ref
      end do
      
      write(nu_sum,'(a)') ' dur file: '//
     :                      f_durations(1:nc_f_durations)
      write(nu_dur,'(
     :   1x,a70, 1x,i5, 
     :   200(1x,es12.5)
     :                                    )') 
     : f_a(1:nc_f_a), num,
     : (t4fractile(i),i=1, nfractiles),
     : (duration(i),i=1, nfractiles),
     : (duration_norm(i),i=1, nfractiles)
     

      do i = 1, nfractiles
         call interpolate_1d(a_sq_int_trap_norm, time, npts, ynull,
     :                          fractile(i), t4fractile_trap(i))
      end do
        
      call interpolate_1d(a_sq_int_trap_norm, time, npts, ynull,
     :                    fractile_durstart, t4fractile_trap_durstart)
      call interpolate_1d(a_sq_int_trap_norm, time, npts, ynull,
     :                    fractile_ref,      t4fractile_trap_ref)
      duration_trap_ref = t4fractile_trap_ref-t4fractile_trap_durstart     
      do i = 1, nfractiles
        duration_trap(i) =  t4fractile(i) -  t4fractile_trap_durstart   
        duration_trap_norm(i) =  duration_trap(i)/duration_trap_ref
      end do
      
      write(nu_sum,'(a)') ' dur_trap file: '//
     :                      f_durations_trap(1:nc_f_durations_trap)
      
      write(nu_dur_trap,'(
     :   1x,a70, 1x,i5, 
     :   200(1x,es12.5)
     :                                    )') 
     : f_a(1:nc_f_a), num,
     : (t4fractile_trap(i),i=1, nfractiles),
     : (duration_trap(i),i=1, nfractiles),
     : (duration_trap_norm(i),i=1, nfractiles)
     
      deallocate (a, a_sq, a_sq_int, a_sq_int_norm, time,
     :               a_sq_int_trap, a_sq_int_trap_norm)

      END DO loop over files

! Close control and output files:

      close(nu_ctl)
      close(nu_sum)
      close(nu_dur)
      close(nu_dur_trap)

      stop
      end
! ------------------------------------------------------------------- SMC2Husid


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\accsqint.for'
      include '\forprogs\accsqint_trapezoidal.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\datetime.for'
      include '\forprogs\rmvtrend.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\dcdt.for'
      include '\forprogs\distaz.for'
      include '\forprogs\interpolate_1d.for'
      include '\forprogs\locate.for'

