! ------------------------------------------------------------------- SMC2PHS
      Program SMC2PHS


! Computes the Fourier amplitude, phase, and phase derivative for specified 
! time series.  

! The output file name has the same stem name as the input file, with "ph"
! as the first two characters of the extension, but without the path 
! (keeping the third letter, which often indicates component).  The exception 
! to not altering the stem name is if the end of the stem name of the input 
! file is "_A" or "_U" and the extension is "smc", in which case the last two 
! characters of the stem name
! is replaced with "ap" or "up" (I had to replace "_" with either "a" or "u"
! because otherwise would have an identical filename for the spectra of
! corrected or uncorrected data, whereas might want them separately).  
! In this case, the extension = "phs" (not "smc, because the file is not
! an smc file).

! Input from a control file; here is an example:
!!Control file for program SMC2PHS
!! As many comment lines as desired, each starting with "!"
!! The string "pp:" indicates a new set of processing parameters
!! to be applied to the following smc files.  The parameters are given on the
!! following 6 lines.  Note that ndec is used only if df_intrp = 0.0.
!!Name of summary file:
! smc2phs.sum
!pp:
!  0.0 200.0 0.0   !tskip, tlength, frctnfrtaper
!  0.0 0.0 25.0 1  ! df_intrp(0=FFT freqs), f_intrp_low, f_intrp_high, ndec
!  0.0 +1.0        !df_smooth(Hz;0=none), signnpw2(<0, backup for npw2, no zpad)
!    .true.        !dc_remove?
!    .false.       !detrend?
!    .true.        !calc_phd?
!  1               ! nsmooth_phd (need an entry even if not used)
! \psv\taiwan99\t0781pea.smc
! \psv\taiwan99\t1293l3v.smc
! \psv\taiwan99\t0781ped.smc
!pp:
!  0.0 90.0 0.05   !tskip, tlength, frctnfrtaper
!  0.05 0.0 50.0 1 ! df_intrp(0=FFT freqs), f_intrp_low, f_intrp_high, ndec
!  0.0 1.0         !df_smooth(Hz;0=none), signnpw2(<0, backup for npw2, no zpad)
!    .true.        !dc_remove?
!    .false.       !detrend?
!    .false.       !calc_phd?
!  1               ! nsmooth_phd (need an entry even if not used)
! file1.smc
! file2.smc
! file3.smc
!stop

! Dates: 05/31/02 - Written by D.M. Boore, patterned after SMC2FS2
!        06/03/02 - Read and use ndec (if df_intrp .eq. 0.0)
!        06/16/02 - Revise output so can easily read npts
!        06/20/02 - call to Ampphphd was altered slightly because
!                   it is now more general.  As part of this generality,
!                   the array passed into ampphphd must be complex
!        07/14/02 - Include the possibility of smoothing the phase derivative
!        07/15/02 - Improved the output
!        07/17/03 - Add sps to smc_npts parameter list
!        10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!        04/24/18 - Replace comment characters * and c with !
!                 - Below ! Compute the Fourier spectra:, changed
!                        do i = 1, npts
!                   to
!                        do i = 1, npw2



!
! Dimension and declaration statements:

      implicit none
      
      real amp(:), amp_out(:), phs(:), phs_out(:)
      real phsd(:), phsd_out(:), freq(:), freq_out(:), ts(:)
      complex ts_c(:)
      allocatable :: amp, amp_out, phs, phs_out
      allocatable :: phsd, phsd_out, freq, freq_out, ts, ts_c

      integer npts, nfreq_out, nc_f_smc
      real tskip, tlength, ftaper
      real df_intrp, f_intrp_low, f_intrp_high
      real df_smooth, signnpw2
      real t_smc, t_fft

      integer nc_f_ctl, nu_ctl, nc_cmnts2skip, nc_f_sum, nu_sum, 
     : nc_buf, ndec, nsmooth_phd, npw2, n4ts_alloc, npts_smc, 
     : nacc_start, nacc_stop, nzpad, i, npw2d2, imode, nfreq, 
     : j, m_start, m_stop, mspct, nc_path, indpsmc, nc_ext_c, 
     : nc_f_phs, indpfs2, nu_phs, iflag 

      real tlength_in, sps, dt, sign, df_fft, df_out, yintrf

      character date_c*8, time_c*11
      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_smc*70, 
     :        f_phs*70, f_sum*40, path*60, c2*2, c2up*2, 
     :        ext_c*10

      character f_ctl*40, colhead(3)*16
      logical f_ctl_exist, f_smc_exist
      logical dc_remove, detrend, calc_phd, apply_afact

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2PHS.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2phs.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call trim_c(f_ctl, nc_f_ctl)

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl,status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')


      write(nu_sum,'(a)') ' Output of program SMC2PHS:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

1000  continue
        buf = ' '
        read(nu_ctl,'(a)',end = 9999) buf
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
        if(buf_upper(1:4) .eq. 'STOP') go to 9999
      
        if(buf_upper(1:3) .eq. 'PP:') then  ! a new set of processing parameters
          read(nu_ctl,*) tskip, tlength_in, ftaper
          write(nu_sum,*)
          write(nu_sum,'(a)') ' A new set of processing parameters:'
          write(nu_sum,'(a)') 
     :       ' tskip, tlength_in, ftaper ='
          write(nu_sum,*) 
     :       tskip, tlength_in, ftaper

          read(nu_ctl,*) df_intrp, f_intrp_low, f_intrp_high, ndec
          write(nu_sum,'(a)') 
     :      ' df_intrp, f_intrp_low, f_intrp_high, ndec = '
          write(nu_sum, *) 
     :        df_intrp, f_intrp_low, f_intrp_high, ndec

          read(nu_ctl,*) df_smooth, signnpw2
          write(nu_sum,'(a)') 
     :      ' df_smooth, signnpw2 = '
          write(nu_sum, *) 
     :      df_smooth, signnpw2

          read(nu_ctl,*) dc_remove
          write(nu_sum,'(a,1x,l1)')  ' dc_remove = ', dc_remove

          read(nu_ctl,*) detrend
          write(nu_sum,'(a,1x,l1)')  ' detrend = ', detrend

          read(nu_ctl,*) calc_phd
          write(nu_sum,'(a,1x,l1)')  ' calc_phd = ', calc_phd

          read(nu_ctl,*) nsmooth_phd
          write(nu_sum,'(a,1x,i3)') ' nsmooth_phd = ', nsmooth_phd

          go to 1000
        end if

        f_smc = ' '
        f_smc = buf(1:nc_buf)
        nc_f_smc = nc_buf     
        inquire(file=f_smc(1:nc_f_smc), exist=f_smc_exist)
        if (.not. f_smc_exist) then
            write(*,'(a)') ' ****** FILE '//f_smc(1:nc_f_smc)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
            go to 1000
        end if

! Passed tests; proceed

        write(nu_sum,*)
        write(nu_sum,'(a)') ' Input file =  '//f_smc(1:nc_f_smc)

! Get number of points on this loop:

        call smc_npts(f_smc(1:nc_f_smc), npts, sps)

        call get_npw2(npts,signnpw2,npw2)

        n4ts_alloc = max0(npts, npw2)

        write(nu_sum, '(a, 3(1x,i5))') ' npts, npw2, n4ts_alloc  = ', 
     :                                   npts, npw2, n4ts_alloc

        allocate( ts(n4ts_alloc), ts_c(n4ts_alloc) )
        allocate( freq(npw2), freq_out(npw2) )
        allocate( amp(npw2), amp_out(npw2) )
        allocate( phs(npw2), phs_out(npw2) )
        allocate( phsd(npw2), phsd_out(npw2) )

! Now read the data from the files:

        write(*,'(1x,a)') 
     :     'Processing file:', f_smc(1:nc_f_smc)

        tlength = tlength_in
        call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :                 ts,  
     :                 char_head, int_head, real_head, comments)
        sps = real_head(2)
        npts_smc = int_head(17)
        dt = 1.0/sps
        t_smc = float(npts_smc-1)*dt
        nacc_start = tskip * sps
        nacc_stop  = nacc_start + tlength * sps

        write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
     :       'signal_in: dt=', dt, 
     :       ' npts_smc=', npts_smc,' t_smc=', t_smc
        write(nu_sum,'(3x,a,1x,f8.2)') 
     :       ' tlength after smcread = ', tlength

! Condition the time series:

        call Condition(ts, dt, npts, npw2, 
     :                 ftaper, dc_remove, detrend, nzpad)      

! Compute the Fourier spectra:

!        do i = 1, npts
        do i = 1, npw2      ! 04/24/18
          ts_c(i) = cmplx(ts(i), 0.0)
        end do

        apply_afact = .true.
        sign = +1.0
        call AmpPhPhD(ts_c, dt, npw2, 
     :    amp, phs, phsd, calc_phd, nsmooth_phd,
     :    apply_afact, df_fft, sign)

        t_fft = float(npw2-1)*dt
        
        write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)')
     :     'signal_fft: dt=', dt, ' npw2=', npw2,' t_fft=', t_fft

! Smooth the amplitude spectra

        npw2d2 = npw2/2
        imode = 1        ! smooth amplitudes
        call SMTHS (amp, npw2d2, df_fft, df_smooth, imode)
!! used to be npw2 in this subroutine

! Set up the frequency array:

! First set up the frequency array for the spectra:

        nfreq = npw2d2
        do j = 1, nfreq
          freq(j) = float(j-1)*df_fft
        end do

        if (df_intrp .eq. 0.0) then
! use fft frequencies, between f_intrp_low and f_intrp_high
          call locate(freq, nfreq, f_intrp_low, m_start)
! DEBUG
          write(nu_sum,'(a)') ' f_intrp_low, m_start, freq(m_start) = '
          write(nu_sum,*)  f_intrp_low, m_start, freq(m_start)
! DEBUG

!          m_start = m_start + 1 ! commented out on 6/15/01

          call locate(freq, nfreq, f_intrp_high, m_stop)
! DEBUG
          write(nu_sum,'(a)') ' f_intrp_high, m_stop, freq(m_stop) = '
          write(nu_sum,*)  f_intrp_high, m_stop, freq(m_stop)
! DEBUG

!          m_stop = m_stop - 1   ! commented out on 6/15/01

          mspct = (m_stop - m_start + 1)/ndec
          do j = 1, mspct
            freq_out(j) = freq(j*ndec + m_start - 1)
          end do
          df_out = df_fft
        else
          mspct = (f_intrp_high-f_intrp_low)/df_intrp + 1
          do j = 1, mspct
            freq_out(j) = float(j-1) * df_intrp + f_intrp_low
          end do
          df_out = df_intrp
        end if

        nfreq_out = mspct

! Interpolate to specified frequencies:

        do j = 1, nfreq_out
          amp_out(j) = yintrf(freq_out(j), freq, amp, nfreq)
          phs_out(j) = yintrf(freq_out(j), freq, phs, nfreq)
          phsd_out(j) = yintrf(freq_out(j), freq, phsd, nfreq)
        end do

        deallocate( ts, ts_c, amp, phs, phsd, freq )

! Write output:

! Construct output filename:
        call get_path_from_file_name(f_smc,nc_f_smc,path,nc_path)
        indpsmc = index(f_smc,'.')
        ext_c = ' '
        ext_c = f_smc(indpsmc+1:nc_f_smc)
        call trim_c(ext_c, nc_ext_c)
        call upstr(ext_c)
        c2 = ' '
        c2 = f_smc(indpsmc-2:indpsmc-1)
        c2up = c2
        call upstr(c2up)

        f_phs = ' '
        f_phs = f_smc(nc_path+1:indpsmc)//'ph'//
     :          f_smc(indpsmc+3:indpsmc+3)

        call trim_c(f_phs,nc_f_phs)

        indpfs2 = index(f_phs,'.')
        if ((c2up .eq. '_A' .or. c2up .eq. '_U') .and. 
     :        ext_c(1:nc_ext_c) .eq. 'SMC') then
          f_phs(indpfs2-2:indpfs2-1) = c2(2:2)//'p'
          f_phs(indpfs2+1:indpfs2+3) = 'phs'
        end if
      
        call trim_c(f_phs,nc_f_phs)

        write(*,'(1x,a)') 
     :     'Writing output to file:', f_phs(1:nc_f_phs)
        write(nu_sum,'(2a)') ' Output file = ', f_phs(1:nc_f_phs)

        call get_lun(nu_phs)
        nu_phs = 10
        open(unit=nu_phs,file=f_phs,status='unknown')

        write(nu_phs,'(a)') '! Output of program SMC2PHS'
        write(nu_phs,'(a)') '! working on file '//f_smc(1:nc_f_smc)
        write(nu_phs,'(a)') 
     :                '! using control file '//f_ctl(1:nc_f_ctl)

        iflag = 0
        call datetime(date_c, time_c, iflag)
        write(nu_phs,'(a)') '!   file written on '//date_c//
     :                      ' at time '//time_c

        write(nu_phs, '(a,1x,f7.2,1x,f7.2,1x,f4.1)') 
     :      '!   tskip, tlength_in, ftaper = ',
     :      tskip, tlength_in, ftaper

        write(nu_phs, '(a, 1x,f3.1,1x,f7.3,1x,f7.3)') 
     :    '!   df_intrp(0=FFT freqs), f_intrp_low, f_intrp_high = ',
     :         df_intrp, f_intrp_low, f_intrp_high

        write(nu_phs, '(a,1x,f5.2,1x,f4.1)') 
     :    '!   df_smooth(Hz;0=none), '//
     :    'signnpw2(<0= backup for npw2, no zpad) = ',
     :      df_smooth, signnpw2

        write(nu_phs, '(a)') 
     :    '!   npw2, nzpad, dt, df_out, nfreq_out = '
!        write(nu_phs, '(1x,i6,1x,i5, 1x,f8.5, 1x,f8.5)') 
         write(nu_phs,*)
     :         npw2, nzpad, dt, df_out, nfreq_out

        write(nu_phs, '(a, 3(1x,l1))') 
     :    '!   dc_remove, detrend, calc_phd = ',
     :      dc_remove, detrend, calc_phd

        do i = 1, 3
          colhead(i) = '________________'
          colhead(i)(5:nc_f_phs+4) = f_phs(1:nc_f_phs)
        end do
        colhead(1)(1:4) = 'amp:'      
        colhead(2)(1:4) = 'phs:'      
        colhead(3)(1:4) = 'phd:'      

        write(nu_phs,'(9x,a,1x,a,1x,a,1x,a)') 
     :         'freq', (colhead(i),i=1,3)

        do j = 1, nfreq_out
          write(nu_phs,'(1p,1x,e12.5,3(5x,e12.5))')
     :          freq_out(j), amp_out(j), phs_out(j), phsd_out(j)
        end do

        close(nu_phs)

        deallocate (amp_out, phs_out, phsd_out, freq_out)
      go to 1000

9999  continue

! Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
! ------------------------------------------------------------------- SMC2PHS

      include '\forprogs\smc2phs_util_subs.for'
      
!      include '\forprogs\fork.for'
!      include '\forprogs\tprfrctn.for'
!      include '\forprogs\dcdt.for'
!      include '\forprogs\skip.for'
!      include '\forprogs\skipcmnt.for'
!      include '\forprogs\get_lun.for'
!      include '\forprogs\upstr.for'
!      include '\forprogs\smc_npts.for'
!      include '\forprogs\smcread.for'
!      include '\forprogs\imnmax.for'
!      include '\forprogs\trim_c.for'
!      include '\forprogs\rc_subs.for'
!      include '\forprogs\smths.for'
!      include '\forprogs\locate.for'
!      include '\forprogs\yintrf.for'
!      include '\forprogs\get_npw2.for'
!      include '\forprogs\zeropad.for'
!      include '\forprogs\conditn.for'
!      include '\forprogs\ampphphd.for'
!      include '\forprogs\smooth_n.for'
!      include '\forprogs\get_path.for'
!      include '\forprogs\datetime.for'


