* ---------------------------------------------------------------- smc2rs
      Program SMC2RS_Sel_Per

* Reads an SMC filename from a control file, computes RS for selected periods,
* and writes the result to a column file, one record per line, along with
* the lat & long of the station.

 
* The next line is the first line in a sample control file:
*!Control file for program smc2rs_sel_per
*!Output file name:
* smc2rs_sel_per.out
*!Effective "epicenter" lat, long for distance computation:
* 42.0, 143.20
*!Effective depth:
* 50
*!Enter number of periods for response spectra:
* 2
*!Periods for response spectra:
* 0.2 1.0
*!Damping value:
* 0.05
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -10.0  
*!accel_file ("STOP" in any column, any case, to quit)
* file1_in.smc  
* file2_in.smc 
* file3_in.smc 
*StoP
 

* Dates: 04/01/05 - Written by D. Boore, based on SMC2RS.
*                   Allow one damping value, and no specification
*                   of record start and stop times (add this to
*                   a later version).
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        02/11/09 - Correct error in t4mean being reset from its input
*                   values when processing a list of files.  The problem is that if a file with
*                   a shorter record length is processed before one with a longer record
*                   length, a mean supposedly determined for the whole record was determined
*                   only for the length of the shorter preceeding record.

 
 
      character char_head(11)*80, comments(200)*80, buf_out*80
      character cmnts2skip(20)*80, stacode*30
      integer int_head(48)
      
      real real_head(50)
      real a(:)
      allocatable :: a
      character f_out*40
      character f_smc*40, buf*79, buf4*4
      real sd(10), psv(10), psa(10), damp, period(10)
      real omega, omega2
      character header(26)*12, damp_c*2

      character f_ctl*40
      logical f_ctl_exist 

      twopi = 2.0*4.0*atan(1.)

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc2rs_sel_per.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc2rs_sel_per.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out, nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

      write(nu_out,'(a)') ' Output file for program SMC2RS_Sel_Per'
      write(nu_out,'(2a)') '   Using control file: ', f_ctl
     
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_out,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) eplat, eplong
      write(nu_out,*) eplat, eplong 

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_out,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) deff
      write(nu_out,*) deff

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_out,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nperiods 
      write(nu_out,*) nperiods 

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_out,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) (period(i),i=1,nperiods)       
      write(nu_out,*) (period(i),i=1,nperiods)       

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_out,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) damp
      write(nu_out,*) damp

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_out,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_out,*) t4mean_in 
      
* Write column headers:

      write(nu_out,'(36x,a, 24x,a, 3x,a, 3x,a, 4x,a,
     :  1x, 10(6x,f5.2, 8x,a, 8x,a))')
     :  'f_smc', 'stacode', 'stalat', 'stalong', 'dist',
     :  (period(i), 'psv', 'paa', i = 1, nperiods)
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

1000  continue

      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf,nc_buf)
      buf4 = ' '
      buf4 = buf(1:4)
      call upstr(buf4)
      if(buf4(1:4) .eq. 'STOP') then
         close(nu_ctl)
         close(nu_out)
         stop
      end if

      f_smc = ' '
      f_smc = buf(1:nc_buf)
      call trim_c(f_smc,nc_f_smc)      
 
* Get npts and allocate the array:

      call smc_npts(f_smc(1:nc_f_smc), npts, sps)
      write(*,'(a,1x,i6)') ' After smc_npts, npts = ', npts
      
 
      allocate (a(npts))

* Read the input file:

      do i = 1, npts
        a(i) = 0.0
      end do

      tskip = 0.0
      tlength = -1.0
      
      call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :               a, char_head, int_head, real_head, comments)
     
      stacode = ' '
      stacode = char_head(3)(2:7)
      call trim_c(stacode,nc_stacode)
      if (nc_stacode .eq. 0) then
        stacode = 'null'
        call trim_c(stacode,nc_stacode)
      end if
      
      
      sps = real_head(2)
      dt = 1.0/sps
      ncomments = int_head(16)
      tend = float(npts-1)/sps

      rnull = real_head(1)
      
      stalat = real_head(11)
      stalong = real_head(12)
      
      if(stalat .eq. rnull .or. stalong .eq. rnull) then
        rdist = -999.0
      else
        call distaz( -1.0, eplat,eplong,stalat,stalong,
     :    rdeg, rkm, az, baz)
        rdist = sqrt(rkm**2+deff**2)
      end if
  
 
!      write(nu_out,'(a)') ' sps, ncomments, npts_in, npts = '
!      write(nu_out, *) sps, ncomments, npts_in, npts

* Step 2: Remove a mean from the whole record

      t4mean = t4mean_in

*  Determine the appropriate mean:
      if(t4mean .lt. 0.0) then  ! do not remove a mean
        avg = 0.0
      else
        if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
          t4mean = tend  ! use whole record
        end if
        n_t4mean = sps * t4mean + 1.1
        cum_avg = 0.0
        do i = 1, n_t4mean
          cum_avg = cum_avg + dble(a(i))
        end do
        avg = cum_avg/float(n_t4mean)
*        avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
        do i = 1, npts
          a(i) = a(i) - avg
        end do
      end if

* Step 3: compute response spectra

 
 
* Loop over periods and damping, computing and writing response spectra:
      do iper = 1, nperiods
        buf_out = ' '
        buf_out = '+ Patience! Computing rd for ts '//
     :    f_smc(1:nc_f_smc)//' and period '
        write(buf_out(60:67),'(f8.3)') period(iper) 
        write(*,'(a)') buf_out
        omega = twopi/period(iper)
        omega2 = omega**2
        d0=0.0
        v0=0.0
        call rdrvaa(a, npts, omega, 
     :                   damp, dt, 
     :                   sd(iper), rv, aa, d0, v0)
        psv(iper) = omega*sd(iper) 
        psa(iper) = omega2*sd(iper) 
      end do ! loop over nperiods
      deallocate (a)
      
      write(nu_out,'(1x,a40, 1x,a30, 
     :   1x,f8.4, 1x,f9.4, 1x,f7.2,
     :   1p, 1x, 10(3e11.3))')     
     :   f_smc(1:nc_f_smc), stacode(1:nc_stacode), 
     :   stalat, stalong, rdist,
     :   (sd(i), psv(i), psa(i), i = 1, nperiods) 
  

      goto 1000 ! go back for another smc file

      end
* ---------------------------------------------------------------- smc2rs

      include '\forprogs\rdrvaa.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\distaz.for'


