* -------------------------------------------------- smc_psagm_vs_r
      Program smc_psagm_vs_r

* Reads time series, computes PSA, and writes into
* a file for plotting vs. r

* NOTE: This program reads pairs and computes the geomean

* The program determines if the component is horizontal, so
* all files in a folder can be included in the list.

* The program computes 3d distances, using station lat, long
* from the smc file

* NOTE:  The file looks at char_head(1)(1:1) to see if the file is
* an acceleration, velocity, or response spectral file, and acts 
* appropriately.  I was tempted to simply compute the velocity and
* response spectrum on the fly, but if the data are from USGS or CGS,
* then the pads have been stripped from the data and I might have
* distorted results.

* If response spectra only, then do not ask for periods -1 and 0 (the
* program checks for this, however).

* Here is a sample control file:
*!Control file for program SMC_PSA_Vs_R
*!Name of summary file:
* smc_psagm_vs_r.sum
*!Eqnum
* 999
*!Mag of quake
* 6.0
*!Number of Fault Planes:
*1
*!Fault orientation (ref lat,long,elev(m),depth, strike, dip, s1, s2, w1, w2):
* 36.03952 -120.60191 0.0 0.0 136.8 83.0 0.0 38.0 0.0 13.0
*!Name of output file:
* park04_bdsn_psagm_vs_r.asc
*!rjb min for plot
* 0.1
*!Damping value (need as placeholder even if are only reading
*!and computing acceleration and/or velocity):
* 0.05
*!Number of periods:
*  6
*!Periods (<0 for pgv, 0 for pga):
* -1.0
* 0.0
* 0.1
* 0.2
* 1.0
* 3.0
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -1000.0  
*! List of files to process (pairs of horizontal components):
* 14094992.BK.PKD.HLE.sac.smc.a0p1.a.pad
* 14094992.BK.PKD.HLN.sac.smc.a0p1.a.pad
*stop
  
* Dates: 11/27/07 - Written by D. Boore, patterned after  smc_psa_vs_r.for
*        02/10/09 - Correct error in t4mean being reset from its input
*                   values when processing a list of files.  The problem is that if a file with
*                   a shorter record length is processed before one with a longer record
*                   length, a mean supposedly determined for the whole record was determined
*                   only for the length of the shorter preceeding record.
 
 
      parameter(nper_max = 300)
      real per_smc(nper_max), psa_smc(nper_max,2), psa_out(10,1000)
      real rd_smc, damp_smc
      real tprd(nper_max,2), tprv(nper_max,2), tpaa(nper_max,2)
      
      real stalat(1000, 2), stalong(1000, 2), 
     :     vmin(2), vmax(2), amin(2), amax(2)

      real alat_ref(10), along_ref(10), elev_ref(10), h_ref(10), 
     :     strike_f(10), dip_f(10), s1(10), s2(10), w1(10), w2(10),
     :     d_jb_segs(10), d_cd2f_segs(10), d_c_segs(10)
     
      real v30rec(1000), d_jb4plot(1000), d_jb(1000), 
     :     d_cd2f4plot(1000), d_cd2f(1000)
     

      character f_stainfo*80, stacode(1000,2)*8, 
     :          buf*120, buf4*4, per_c*5
      character f_smc(2)*80, f_sum*80, f_out*80, colhead(10)*8, 
     :          frmt_c*120
      character buf_out*80, calc_rs*20
      
      integer iclass, nc_stacode(1000,2), nc_f_smc(2)
      
      character char_head(11)*80, comments(500)*80, cmnt(50)*80
      integer int_head(48)
      real real_head(50)

      real a(:), vel(:), dis(:), y(:)
      allocatable :: a, vel, dis, y
      real sd(10), psv(10), psa(10),  
     :     damp_ctl, per_ctl(10), per_out(10),
     :     ampcorr(10, 1000)
      real omega, omega2
 
      character f_ctl*40
      logical f_ctl_exist, f_smc_exist
      logical rmv_trnd

      twopi = 2.0*4.0*atan(1.)

      call get_lun(nu_debug)
      open (unit=nu_debug, file='debug.out', 
     :      status='unknown')

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file '//
     :    '(cr=smc_psagm_vs_r.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') 
     :      f_ctl = 'smc_psagm_vs_r.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')


*  Get the name of summary file:

      call skipcmnt(nu_ctl, cmnt, ncmnt)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      write(*,'(2a)') ' f_sum = ', f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open (unit=nu_sum, file=f_sum(1:nc_f_sum), 
     :      status='unknown')
  
      
*  Get the eqnum (usually from NGA flatfile):
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read(nu_ctl,*) ieqnum
       
*  Get the mag of quake (usually from NGA flatfile):
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read(nu_ctl,*) amag_ctl
       
*  Get fault segment information:
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read(nu_ctl,*) n_f
 
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      do i = 1, n_f
        read(nu_ctl,*) alat_ref(i), along_ref(i), elev_ref(i), h_ref(i),
     :                 strike_f(i), dip_f(i), 
     :                 s1(i), s2(i), w1(i), w2(i)
      end do

*  Get the name of output file:

      call skipcmnt(nu_ctl, cmnt, ncmnt)
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out, nc_f_out)
      write(*,'(2a)') ' f_out = ', f_out(1:nc_f_out)
      write(nu_sum,'(2a)') ' f_out = ', f_out(1:nc_f_out)
      call get_lun(nu_out)
      open (unit=nu_out, file=f_out(1:nc_f_out), 
     :      status='unknown')
     
*  Get minimum d_jb for plot:
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read(nu_ctl,*) d_jbmin
       
* Get damping:
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read(nu_ctl,*) damp_ctl
      
* Get number of periods (<= 10):
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read(nu_ctl,*) nper_ctl

* Get periods:
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      do i = 1, nper_ctl
        read(nu_ctl,*) per_ctl(i)
      end do
      
* Get t4mean:
      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read(nu_ctl,*) t4mean_in
       
* Loop over file names:

      call skipcmnt(nu_ctl, cmnt, ncmnt)
      
      nsta = 0
      
1000  continue
      do i = 1, 2
        buf = ' '
        read(nu_ctl,'(a)',end = 9999) buf
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(buf4(1:4) .eq. 'STOP') go to 9999
      
        f_smc(i) = ' '
        f_smc(i) = buf(1:nc_buf)
        nc_f_smc(i) = nc_buf     
        inquire(file=f_smc(i)(1:nc_f_smc(i)), exist=f_smc_exist)
 
        if (.not. f_smc_exist) then
          write(nu_sum,'(a)') ' ****** FILE '//
     :           f_smc(i)(1:nc_f_smc(i))// 
     :          ' DOES NOT EXIST; QUITTING!!! ******* '
          close(nu_ctl)
          close(nu_sum)
          stop
        end if

* Get various information from headers:

        write(nu_sum, '(a)') ' Processing file '//
     :                     f_smc(i)(1:nc_f_smc(i)) 

        call ReadHdrs(f_smc(i)(1:nc_f_smc(i)), 
     :                char_head, int_head, real_head, comments)

        inull = int_head(1)
        rnull = real_head(1)
      
* Skip if vertical component:
      
        if (int_head(13) .ne. inull .and. 
     :    int_head(13) .ne. 90) then     ! must be vertical
          write(nu_sum, '(3x,a,1x,i5)') 'int_head(13) = ', int_head(13)
          write(nu_sum, '(3x,a)') 
     :        ', not a horizontal component; SKIPPING to next record'
          go to 1000
        end if

* Skip if station coordinates are missing:

        stalat(nsta+1,i) = real_head(11)
        stalong(nsta+1,i) = real_head(12)
        if (stalat(nsta+1,i)  .eq. rnull .or.
     :    stalong(nsta+1,i) .eq. rnull) then
          write(nu_sum, '(3x,a,1p,2(1x,e10.3))') 
     :    'stalat, stalong = ', stalat(nsta+1,i), stalong(nsta+1,i)
          write(nu_sum, '(3x,a)') 
     :        ', a null value; QUITTING'
          close(nu_ctl)
          close(nu_sum)
          stop
        end if

* Get npts and allocate the array:

        call smc_npts(f_smc(i)(1:nc_f_smc(i)), npts, sps)
        write(nu_sum,'(a,1x,i6)') '   After smc_npts, npts = ', npts
      
 
        allocate (a(npts), vel(npts), dis(npts))

* Read the input file:

        do j = 1, npts
          a(j) = 0.0
        end do

        tskip = 0.0
        tlength = -1.0
      
        call SMCRead(f_smc(i)(1:nc_f_smc(i)), tskip, tlength,
     :               a, char_head, int_head, real_head, comments)
     
      
        stacode(nsta+1,i) = ' '
        stacode(nsta+1,i) = char_head(3)(1:8)
        call trim_c(stacode(nsta+1,i),nc_stacode(nsta+1,i))
        if (nc_stacode(nsta+1,i) .eq. 0) then
          stacode(nsta+1,i) = 'null'
          call trim_c(stacode(nsta+1,i),nc_stacode(nsta+1,i))
        end if
          
      
        sps = real_head(2)
        dt = 1.0/sps
        ncomments = int_head(16)
        tend = float(npts-1)/sps
        
        write(nu_sum,'(a)') '  sps, dt, ncomments, tend, t4mean_in = '
        write(nu_sum, * )      sps, dt, ncomments, tend, t4mean_in
        
        ru = -999.0
      
 
* Step : Remove a mean from the whole record

        t4mean = t4mean_in

*  Determine the appropriate mean:
        if(t4mean .lt. 0.0) then  ! do not remove a mean
          avg = 0.0
        else
          if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
            t4mean = tend  ! use whole record
          end if
          n_t4mean = sps * t4mean + 1.1
          cum_avg = 0.0
          do j = 1, n_t4mean
            cum_avg = cum_avg + dble(a(j))
          end do
          avg = cum_avg/float(n_t4mean)
*        avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
         write(nu_sum,'(a,1x,1p,e10.3)') ' avg = ', avg
          do j = 1, npts
            a(j) = a(j) - avg
          end do
        end if

* Step: compute response spectra, pgv, pga
 
* Loop over periods and damping, computing response spectra for one component:
        do iper = 1, nper_ctl
        

          per_out(iper) = per_ctl(iper)

          write(*,'(a,1x,i2,1x,f7.3)') ' iper, per_out(iper) = ',
     :                                    iper, per_out(iper)

          buf_out = ' '
          buf_out = '+ Patience! Computing rd for ts '//
     :      f_smc(i)(1:nc_f_smc(i))//' and period '
          write(buf_out(60:67),'(f8.3)') per_ctl(iper) 
          write(*,'(a)') buf_out
          if (per_ctl(iper) .lt. 0.0) then  ! pgv
            v0 = 0.0
            d0 = 0.0
            rmv_trnd = .false.
            call acc2vd(a, npts, dt, rmv_trnd, v0, d0, vel, dis)
            call abs_mnmaxidx(vel, 1, npts, 1,
     :        vmin(i), vmax(i), indxpgv_min, indxpgv_max, 
     :        psa_smc(iper, i)) 
     
            write(nu_sum, '(a,1x,1p,e10.3)') 
     :        ' pgv = ', psa_smc(iper,i)
            
          else if (per_ctl(iper) .eq. 0.0) then  ! pga
            call abs_mnmaxidx(a, 1, npts, 1,
     :             amin(i), amax(i), indxpga_min, indxpga_max, 
     :             psa_smc(iper, i)) 

            write(nu_sum, '(a,1x,1p,e10.3)') 
     :        ' pga = ', psa_smc(iper, i)
            
          else ! rs
            omega = twopi/per_ctl(iper)
            omega2 = omega**2
            d0=0.0
            v0=0.0
            call rdrvaa(a, npts, omega, 
     :                   damp_ctl, dt, 
     :                   rd_smc, rv, aa, d0, v0)
            psa_smc(iper, i) = omega2*rd_smc 
            
            write(nu_sum, '(a,1x,f7.3,1p,1x, e10.3)') ' per, psa = ', 
     :         per_out(iper), psa_smc(iper, i)
            
          end if
          
        end do ! loop over nperiods
      
        nper_out = nper_ctl
        
        deallocate (a, vel, dis)
        
      end do  ! end loop over components
      
* Check for equality of stalat, stalong:

      if (stalat(nsta+1,1) .ne. stalat(nsta+1,2) .or. 
     :    stalong(nsta+1,1) .ne. stalong(nsta+1,2)) then
         print *,' ERROR, stalat or stalong not same;'
         print *,' file 1 = ', f_smc(1)(1:nc_f_smc(1))
         print *,' file 2 = ', f_smc(2)(1:nc_f_smc(2))
         print *,' QUITTING!!!'
         close(nu_ctl)
         close(nu_sum)
         stop
      end if
      
      nsta = nsta + 1

* Loop over periods, computing and writing geometric mean response spectra:

      do iper = 1, nper_ctl        
        psa_out(iper, nsta) = sqrt(psa_smc(iper,1)*psa_smc(iper,2))
      end do
        
* Compute distance      
       
      staelev = 0.0
      h_min_c = 3.0
      
      call dist3dmf(stalat(nsta,1), stalong(nsta,1), staelev,
     : alat_ref, along_ref, elev_ref, h_ref, strike_f, dip_f,
     : w1, w2, s1, s2, n_f, h_min_c, d_jb(nsta), az_jb, 
     : d_cd2f(nsta), az_cd2f, 
     : d_c, az_c, d_jb_segs, d_cd2f_segs, d_c_segs)
     
      if (d_jb(nsta) .lt. d_jbmin) then
        d_jb4plot(nsta) = d_jbmin
      else
        d_jb4plot(nsta) = d_jb(nsta)
      end if

      if (d_cd2f(nsta) .lt. d_jbmin) then
        d_cd2f4plot(nsta) = d_jbmin
      else
        d_cd2f4plot(nsta) = d_cd2f(nsta)
      end if


* Go back for a new file:
     
      go to 1000

9999  continue

      write(nu_out,'(a)') '!Output of program smc_psagm_vs_r.for'
      write(nu_out,'(a)') '!using control file: '//f_ctl(1:nc_f_ctl)
      write(nu_out,'(a)') '!and summary file: '//f_sum(1:nc_f_sum)
      write(nu_out,'(a)') '!nper, periods:'
      write(nu_out, '(2x,i5)') nper_out
      write(nu_out, '(1x,10(1x,f7.3))') (per_out(i),i=1,nper_out)
      


* Set up and write column headers of output file:

      do i = 1, nper_out
        per_c = ' '
        write(per_c,'(f5.2)') per_out(i)
        call trim_c(per_c,nc_per_c)
        colhead(i) = ' '
        write(colhead(i)(1:8),'(a8)') 'PSA00000'
        colhead(i)(8-nc_per_c+1:8) = per_c(1:nc_per_c)
        write(nu_debug,'(a,i2,a)') ' colhead(', i, ') = '//colhead(i)
      end do
      
      frmt_c = ' '
      frmt_c = 
     : '(1x,a, 2x,a, 2x,a,'// 
     :   '3x,a, 3x,a,'//
     :   '1x,a, 3x,a, 2x,a, xx(3x,a,5x,a))' 
      call trim_c(frmt_c,nc_frmt_c)
      write(frmt_c(48:49), '(i2)') nper_out
      call trim_c(frmt_c,nc_frmt_c)
      
      write(nu_sum,'(a)') ' frmt_c='//frmt_c(1:nc_frmt_c)


      write(nu_out,frmt_c(1:nc_frmt_c)) 
     :   'eqnum', 'mag', 'stacode', 
     :   'stalat','stalong',
     :   'iclass', 
     :    'd_jb', 'dcd2f',
     :    (colhead(i), 'logPSA', i = 1, nper_out)
  

* Write results to output file

      iclass = 0
      
      do j = 1, nsta
        write(nu_out,'(
     :        1x,i5, 1x,f4.2, 1x,a8,
     :        1x,f8.4, 1x,f9.4,
     :        6x,i1, 
     :        1x,f6.1, 1x,f6.1, 
     :        10(1p,1x,e10.3,1x,e10.3))') 
     :        ieqnum, amag_ctl, stacode(j,1)(1:nc_stacode(j,1)), 
     :        stalat(j,1), stalong(j,1),
     :        iclass,
     :        d_jb4plot(j), d_cd2f4plot(j), 
     :        ( psa_out(i,j), 
     :          alog10(psa_out(i,j) ), i = 1, nper_out )
      end do
      
* Close control and output files:

      close(nu_ctl)
 
      close(nu_out)
      close(nu_sum)
      close(nu_debug)
      
      stop

      end
* -------------------------------------------------- smc_psa_vs_r

 

      include '\forprogs\acc2vd.for'
      include '\forprogs\Trapezoidal_Integration.for'
      include '\forprogs\dist3dmf.for'
      include '\forprogs\rmvtrend.for'     
      include '\forprogs\rdrvaa.for'
      include '\forprogs\abs_mnmaxidx.for'     
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\rs2Rdhdr.for'
      include '\forprogs\rs2Read.for'
      include '\forprogs\fs2Read.for'
      include '\forprogs\readhdrs.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\yintrf.for'
      include '\forprogs\BJF94V30Amp.for'      
      include '\forprogs\lin_interp.for'      
      include '\forprogs\locate.for'      
 
* ----------------------- BEGIN RCC -----------------------------------
*      subroutine RCC (KPARM,CSTR_IN,NCSTR_IN,C,NC,ibgn,iend)

* --------------------------- BEGIN RCF ------------------------------
*      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)

* -------------------------- BEGIN RCI -----------------------
*      subroutine RCI (KPARM,CSTR_IN,NCSTR_IN,I,ibgn,iend,ISTAT)

      include '\forprogs\rcc.for'
      include '\forprogs\rcf.for'
      include '\forprogs\rci.for'
      include '\forprogs\tabs_rmv.for'

      include '\gm_predictions\bjf97.for'

  
