* ------------------------------------------------------------------- SMC_V2A
      Program SMC_V2A


* Produces files of acceleration from velocity 

* The output file name has the same stem name as the input file, with "a" 
* appended to the name. The position of "a" is determined from
* the control file.

* Input from a control file; here is an example:
*!Control file for program SMC_V2A
*! As many comment lines as desired, each starting with "!"
*!Name of summary file:
* smc_v2a.sum
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -1.0 
*!itype_diff_op (3=centered three-point; 5=centered five-point):
* 3
*!List of files 
* hm51301.xvc
*Stop

* Dates: 08/11/02 - Written by D.M. Boore, patterned after SMC2VD
*        10/21/02 - Added stuff to comment headers
*        07/17/03 - Add sps to smc_npts parameter list
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        02/10/09 - Correct error in t4mean being reset from its input
*                   values when processing a list of files.  The problem is that if a file with
*                   a shorter record length is processed before one with a longer record
*                   length, a mean supposedly determined for the whole record was determined
*                   only for the length of the shorter preceeding record.
*        04/29/09 - Output file name is the same as the input name, with ".a1", ".a2", or ".a3"
*                   appended to end, where the number is the itype.
*                   The original algorithm I used (based on straightline segments
*                   in velocity) works poorly, so it is no longer an option.
*        06/12/09 - itype had two meanings.  The type of difference operator is now called
*                   itype_diff_op
*        06/13/09 - Let itype_diff_op equal the number of points in the differentiation operator.
*        06/15/09 - Adopt the SAC procedure of accounting for undefined
*                   acceleration values by reducing the number of points (this must be done 
*                   in v2a, but I need to call v2a with npts_v and npts_a).
*                   Note that I have NOT changed the time of the first sample, as this
*                   is given by integer headers with day, hr, min, sec, msec.  To change the time
*                   properly means checking for these values being defined, and then
*                   converting to a fractional julian day, adding dt or 2 dt (for the 3 and 5 pt
*                   differentiators), and then converting back to day, hr, min, sec, msec.  
*                   The possibility must be allowed that only the day is undefined.  
*                   
 
      character precision*8


c
c Dimension and declaration statements:

      real a(:), v(:)
      allocatable :: a, v

      integer npts, nc_f_a, nc_f_v, itype_diff_op
      real t4mean
      real tend

      character date*8, time*10 

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_sum*40, f_a*70, f_v*70

      character f_ctl*40
      logical f_ctl_exist, f_v_exist, rmv_trnd

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC_V2A.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc_v2a.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC_V2A:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) t4mean_in
      write(nu_sum,'(a,1x,f9.2)') '   t4mean_in = ', t4mean_in

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) itype_diff_op
      write(nu_sum,'(a,1x,i1)') '   itype_diff_op = ', itype_diff_op

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      num = 0
      
1000  continue
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
      if(buf_upper(1:4) .eq. 'STOP') go to 9999
      
      f_v = ' '
      f_v = buf(1:nc_buf)
      nc_f_v = nc_buf     
      inquire(file=f_v(1:nc_f_v), exist=f_v_exist)
      if (.not. f_v_exist) then
          write(*,'(a)') ' ****** FILE '//f_v(1:nc_f_v)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

* Passed tests; proceed

      num = num + 1

* Get number of points on this loop:

      call smc_npts(f_v(1:nc_f_v), npts, sps)

*      write(nu_sum,*)
*      write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

      allocate( a(npts+5), v(npts+5) )

* Now read the data from the files:

      write(*,'(1x,a)') 
     :   'Processing file:', f_v(1:nc_f_v)

      tskip = 0.0
      tlength = -1.0

      call SMCRead(f_v(1:nc_f_v), tskip, tlength,
     :               v,  
     :               char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      rnull = real_head(1)
      v0 = 0.0
      d0 = 0.0

      sps = real_head(2)
      ncomments_v = int_head(16)
      npts_smc = int_head(17)
      dt = 1.0/sps
      tend = float(npts_smc-1)*dt

*      write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
*     :     'signal_in: dt=', dt, 
*     :     ' npts_smc=', npts_smc,' tend=', tend

* Step 1: Remove a mean from the whole record


      t4mean = t4mean_in

*  Determine the appropriate mean:
      if(t4mean .lt. 0.0) then  ! do not remove a mean
        avg = 0.0
      else
        if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
          t4mean = tend  ! use whole record
        end if
        n_t4mean = sps * t4mean + 1.1
        cum_avg = 0.0
        do i = 1, n_t4mean
          cum_avg = cum_avg + dble(v(i))
        end do
        avg = sngl(cum_avg/dble(float(n_t4mean)))
*  Remove the mean from the whole record:
        do i = 1, npts_smc
          v(i) = v(i) - avg
        end do
      end if

* Step2: Differentiate to acceleration

      npts_v = npts_smc

      call v2a(v, npts_v, dt, a, npts_a, itype_diff_op)  ! itype=3 = 3 point, 5 = 5 point

* Step 3: Write output:

* Construct output filename:
      indp = index(f_v,'.')
      f_a = ' '
      if (itype_diff_op .eq. 3) then
        f_a = f_v(1:nc_f_v)//'.a3'
      else if (itype_diff_op .eq. 5) then
        f_a = f_v(1:nc_f_v)//'.a5'
      else
        print *,' invalid itype = ', itype_diff_op, '; QUITTING!!!'
        stop
      end if
        
      
      call trim_c(f_a,nc_f_a)
      
      print *,'Writing Acceleration output to file:', f_a(1:nc_f_a)

* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( date, time )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'

      ncomments = ncomments_v
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| Acceleration, from Processing vel file '//
     :       f_v(1:nc_f_v)//
     :      ' using program SMC_V2A'
      ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1)') 
     :      '|     and operator given by itype_diff_op'//
     :      ' (3= 3 pt; 5 = 5 pt) = ', itype_diff_op
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   on '//date(1:4)//'/'//date(5:6)//'/'//date(7:8)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   at time '//time(1:2)//':'//time(3:4)//':'//time(5:10)
      if (t4mean .lt. 0.0) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
      else
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|     mean determined from t = 0.0 to t = ', t4mean
      end if
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   resulting in file '//f_a(1:nc_f_a)

      ncomments = ncomments + 1
      comments(ncomments) = '|'

      int_head(16) = ncomments 
      itype = 2
      int_head(17) = npts_a
      
      call alter_start_time(int_head, dt, itype_diff_op)

!DEBUG
      y_smallest = 1.0e38
      do i = 1, int_head(17)
        if (abs(a(i)) < y_smallest) y_smallest = abs(a(i))
      end do
      print *,' In smc_v2a, y_smallest = ', y_smallest
!DEBUG

      call SMCWrite(f_a, precision,
     :                a, char_head, itype, 
     :                    int_head, real_head, comments)

      write(nu_sum,'(1x,a, 1x,i3, 1x,a, 1x,i6, 1x,i6, 1x,1p,e10.3)') 
     :        'num, File, npts_v, npts_a, a_last = ',
     :         num, f_a(1:nc_f_a), npts_v, npts_a, a(npts_a)

      deallocate (a, v)

      go to 1000

9999  continue

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC_V2A

* ---------------------------------------------------------- alter_start_time
      subroutine alter_start_time(int_head, dt, itype_diff_op)
      integer int_head(*)
      integer iday, ihr, imin, isec, imsec
      real time
      
      if (itype_diff_op .ne. 3 .and. itype_diff_op .ne. 5) then
        return
      end if
      
      inull = int_head(1)

      do i = 4, 7
        if (int_head(i) .eq. inull) then
          return
        end if
      end do

      iday = int_head(3)
      ihr = int_head(4)
      imin = int_head(5)
      isec = int_head(6)
      imsec = int_head(7)
      time = float(imin)*60.0+float(isec)+float(imsec)/1000.0
      
      if (itype_diff_op .eq. 3) then
     
        time = time + dt
        
      else  if (itype_diff_op .eq. 5) then
      
        time = time + 2.0*dt
        
      else
        print *,' ERROR in value of itype_diff_op'
        stop
      end if
      
      if (time .gt. 3600.0) then
        ihr = ihr + 1
        time = time - 3600.0
      end if

      if (ihr .gt. 24) then
        if (iday .ne. inull) then
          iday = iday + 1
          ihr = ihr - 24
        end if
      end if
      
* Using modulo arithmetic is probably a better way to do this
      rmin = time/60.0
      imin = int(rmin)
      rsec = (rmin - float(imin))*60.0
      isec = int(rsec)
      imsec = int((rsec - float(isec))*1000.0)
       
      int_head(3) = iday
      int_head(4) = ihr
      int_head(5) = imin
      int_head(6) = isec
      int_head(7) = imsec 
       
      return
      end
* ---------------------------------------------------------- alter_start_time
      
      include '\forprogs\distaz.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\v2a.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\rmvtrend.for'
      include '\forprogs\mnmaxidx.for'
