* ------------------------------------------------------------------- SMC_V2AD
      Program SMC_V2AD


* Produces files of acceleration and displacement from velocity file

* Construct output filenames by appending ".a", ".d" to the
* input file name.

* Input from a control file; here is an example:
*!Control file for program SMC_V2AD
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! following 4 lines.
*!Name of summary file:
* smc_v2ad.sum
*!mean info:
*! type of mean (0=no correction, 1=standard average; -1=specify mean)
*! t4mean_strt, t4mean_stop, mean (use if type = -1)
*! NOTE: need entries even if the parameter is not used (because read
*! info use list-directed [unformatted] input) 
*! Note that if working with a processed record, it is
*! inadvisible to remove the mean as this may produce unintended
*! distortions at long periods --- so in most cases set t4mean < 0.0):
* 0  1.0 20.0  -999.
*!itype (1=centered three-point; 2=centered five-point):
*!ignore for now; itype = 2 is assumed
* 2
*!List of files 
* t0781pea.smc
* t0781ped.smc
*Stop

* Dates: 06/28/04 - Written by D.M. Boore, patterned after SMC2VD
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        02/10/09 - Correct error in t4mean_strt and t4mean_stop being reset from their input
*                   values when processing a list of files.  The problem is that if a file with
*                   a shorter record length is processed before one with a longer record
*                   length, a mean supposedly determined for the whole record was determined
*                   only for the length of the shorter preceeding record.
*        04/29/09 - Use 5-point difference operator for the derivative.
*                   The original algorithm I used (based on straightline segments
*                   in velocity) works poorly, so it is no longer an option.

      character precision*8


c
c Dimension and declaration statements:

      real a(:), v(:), d(:)
      allocatable :: a, v, d

      integer npts, nc_f_a, nc_f_v, nc_f_d
      real tend
      real*8 cum_avg

      character date*8, time*10 

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_sum*40, f_a*70, f_v*70, f_d*70, f_out*70

      character f_ctl*40
      logical f_ctl_exist, f_v_exist, rmv_trnd, use_v0_d0

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC_V2AD.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc_v2ad.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC_V2AD:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_sum,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_sum,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :    itype

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      num = 0
      
1000  continue
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
      if(buf_upper(1:4) .eq. 'STOP') go to 9999
      
      f_v = ' '
      f_v = buf(1:nc_buf)
      nc_f_v = nc_buf     
      inquire(file=f_v(1:nc_f_v), exist=f_v_exist)

      if (.not. f_v_exist) then
          write(*,'(a)') ' ****** FILE '//f_v(1:nc_f_v)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

* Passed tests; proceed

      num = num + 1

* Get number of points on this loop:

      call smc_npts(f_v(1:nc_f_v), npts, sps)

      write(nu_sum,*)
      write(nu_sum,'(a,1x,i5)') ' For input file '//f_v(1:nc_f_v)//
     :                    ', npts = ', npts

      allocate( a(npts+5), v(npts+5), d(npts+5) )

* Now read the data from the files:

      write(*,'(1x,a)') 
     :   'Processing file:', f_v(1:nc_f_v)

      tskip = 0.0
      tlength = -1.0

      call SMCRead(f_v(1:nc_f_v), tskip, tlength,
     :               v,  
     :               char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      rnull = real_head(1)
      v0 = 0.0
      d0 = 0.0

      sps = real_head(2)
      ncomments_a = int_head(16)
      npts = int_head(17)
      dt = 1.0/sps
      tend = float(npts-1)*dt

* Step 1: Remove a mean from the whole record

      t4mean_strt = t4mean_strt_in
      t4mean_stop = t4mean_stop_in
      
*  Determine the appropriate mean:
      if (itype_mean .eq. 0) then
        n_t4mean = 0
        avg = 0.0
      else if (itype_mean .eq. -1) then   ! use specified mean
        avg = spec_mean
      else if (itype_mean .eq. 1) then
        if (t4mean_strt .lt. 0.0) t4mean_strt = 0.0
        if (t4mean_stop .gt. tend) t4mean_stop = tend
        n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
        n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
        n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
        t4mean = float(n_t4mean -1)/sps
*        write(nu_sum,'(a,1x,f6.2)') ' t4mean = ', t4mean
        cum_avg = 0.0
        do i = n_t4mean_strt, n_t4mean_stop
          cum_avg = cum_avg + dble(v(i))
        end do
        avg = sngl(cum_avg/n_t4mean)
      else            ! shouldn't be here!
        write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        write(*) '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        deallocate (a, v, d)
        go to 9999
      end if

*  Remove the mean from the whole record:
      do i = 1, npts
        v(i) = v(i) - avg
      end do

* Step2: Differentiate and integrate to obtain acceleration and displacement

      call v2ad(v, npts, dt, a, d)

* Step 3: Write output:

* Write velocity if avg .ne. 0.0:
      if (avg .ne. 0.0) then
        f_out = ' '
        f_out = f_v(1:nc_f_v)//'.v'
        call trim_c(f_out, nc_f_out)

        write(*,'(1x,a)') 
     :   'Writing velocity output to file:', f_out(1:nc_f_out)

* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( date, time )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'


        ncomments = ncomments_a
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| Velocity, from processing vel file '//f_v(1:nc_f_v)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| using program SMC_V2AD'
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   on '//date(1:4)//'/'//date(5:6)//'/'//date(7:8)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   at time '//time(1:2)//':'//time(3:4)//':'//time(5:10)

        if (itype_mean .eq. 0) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
        else if (itype_mean .eq. 1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
        else if (itype_mean .eq. -1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
        else
          write(*,'(a)') ' Not supposed to be here.'
        end if

        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|   Initial velocity = ', v0

        ncomments = ncomments + 1
        comments(ncomments) = '|   resulting in output file '//
     :      f_out(1:nc_f_out)

        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments 
        itype = 3


        call SMCWrite(f_out, precision,
     :                v, char_head, itype, 
     :                    int_head, real_head, comments)

      end if

* Write acceleration:

      f_a = ' '
      f_a = f_v(1:nc_f_v)//'.a'
      call trim_c(f_a, nc_f_a)

      write(*,'(1x,a)') 
     :   'Writing acceleration output to file:', f_a(1:nc_f_a)

* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( date, time )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'

      ncomments = ncomments_a
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| Acceleration, from processing vel file '//f_v(1:nc_f_v)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| using program SMC_V2AD'

      ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1)') 
     :      '|     and centered 5-point finite difference operator'
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   on '//date(1:4)//'/'//date(5:6)//'/'//date(7:8)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   at time '//time(1:2)//':'//time(3:4)//':'//time(5:10)

      if (itype_mean .eq. 0) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
      else if (itype_mean .eq. 1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
      else if (itype_mean .eq. -1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
      else
        write(*,'(a)') ' Not supposed to be here.'
      end if

        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|   Initial velocity = ', v0

      ncomments = ncomments + 1
      comments(ncomments) = '|   resulting in output file '//
     :      f_a(1:nc_f_a)

      ncomments = ncomments + 1
      comments(ncomments) = '|'

      int_head(16) = ncomments 
      itype = 2

      call SMCWrite(f_a, precision,
     :                a, char_head, itype, 
     :                    int_head, real_head, comments)

      f_d = ' '
      f_d = f_v(1:nc_f_v)//'.d'
      call trim_c(f_d, nc_f_d)

* Write displacement:

      write(*,'(1x,a)') 
     :   'Writing displacement output to file:', f_d(1:nc_f_d)

* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( date, time )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'

      ncomments = ncomments_a
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| Displacement, from processing vel file '//f_v(1:nc_f_v)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| using program SMC_V2AD'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   on '//date(1:4)//'/'//date(5:6)//'/'//date(7:8)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   at time '//time(1:2)//':'//time(3:4)//':'//time(5:10)

      if (itype_mean .eq. 0) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
      else if (itype_mean .eq. 1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
      else if (itype_mean .eq. -1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
      else
        write(*,'(a)') ' Not supposed to be here.'
      end if

      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|   Initial displacement = ', d0

      ncomments = ncomments + 1
      comments(ncomments) = '|   resulting in output file '//
     :      f_d(1:nc_f_d)

      ncomments = ncomments + 1
      comments(ncomments) = '|'

      int_head(16) = ncomments 
      itype = 4

      call SMCWrite(f_d, precision,
     :                d, char_head, itype, 
     :                    int_head, real_head, comments)

      deallocate (a, v, d)

      go to 1000

9999  continue

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC2VD


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\v2ad.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\rmvtrend.for'
      include '\forprogs\mnmaxidx.for'

