! ------------------------------------------------------------------- SMCCoher
      Program SMCCoher

!############## WARNING ############################
! specifying different sets of processing parameters (PP) does not work in this version
! Revise using smc2fs2 as a guide.  But doing one set per run works fine.
!############## WARNING ############################


! Computes the coherence between two smc files (the application
! in mind being two components for the same recording).

  
! Writes output in column format.

 
! Input from a control file; here is an example:
!!Control file for program SMC2Coher
!! Revision of program involving a change in the control file on this date:
!   09/07/18
!! As many comment lines as desired, each starting with "!"
!! The string "pp:" indicates a new set of processing parameters
!! to be applied to the following smc files.  The parameters are given on the
!! lines following "pp:", until the next "pp:" line or until "stop" is 
!! encountered.
!! NOTE: Use the tapers with caution, choosing them so that important signal
!! is not reduced by the tapering.  This can be particularly a problem with 
!! analog data from relatively small earthquakes that triggered near the 
!! S-wave arrival.  
!!Name of summary file:
! smccoher.sum
!! -----------------------------------------------------------------------------
!!
!! Meaning of smoothing input parameters
!!
!! NO SMOOTHING  (NOTE: smoothing is necessary in computing coherency; this input is a carry over from the program smc2fs2)
!! itype = 0   
!! SMOOTHING OVER EQUALLY SPACED FREQUENCIES
!! itype = 1: box weighting function
!!   smooth_param = width of box weighting function (Hz)
!! itype = 2: triangular weighting function
!!   smooth_param = width of triangular weighting function (Hz)
!! SMOOTHING OVER LOGARITHMICALLY SPACED FREQUENCIES
!! itype = 3: box weighting function
!!   smooth_param = xi, which is the fraction of a decade for the
!!                  box weighting function 
!! itype = 4: triangular weighting function
!!   smooth_param = xi, which is the fraction of a decade for the
!!                  triangular weighting function 
!! itype = 5: Konno and Ohmachi weighting function (see BSSA 88, 228-241)
!!   smooth_param = xi, which is the fraction of a decade for which
!!                  the Konno and Ohmachi weighting function is greater
!!                  than 0.043.(it is related to
!!                  their smoothing parameter b by the equation
!!                  b = 4.0/smooth_param, so we have this correspondence between
!!                  b and smooth_param
!!                         b smooth_param 
!!                        10         0.40
!!                        20         0.20
!!                        40         0.10
!!                  
!!                  b = 40 seems to be commonly used, but I do not think that it
!!                  gives enough smoothing; I PREFER SMOOTH_PARAM = 0.2, 
!!                  corresponding to b = 20. 
!!
!! ipow = power of FAS to be smoothed (2 = smoothing energy spectrum)
!!
!! df_smooth: Note: need df_smooth for linearly-spaced smoothers, 
!! and generally it should be the df from the fft.  For general x data, it is
!! the spacing between x values, assumed to be constant,  The reason for
!! including it as an input parameter is to "fool" the
!! program to do smoothing over a specified number of points by
!! setting df_smooth = 1 and smooth_param = number of points (including 
!! points with zero weight at ends; e.g., smooth_param = 5 will 
!! give a smoother with weights 0, 1/4, 2/4, 1/4, 0; smooth_param
!! should be odd).
!!
!! -----------------------------------------------------------------------------
!! Meaning of frequency specification parameters:
!!
!!FREQUENCY SPECIFICATION: 
!!    f_low, f_high (use FFT frequencies; see smc2fs2 for a more general specification of frequencies)
!! -----------------------------------------------------------------------------
!!
!pp: new set of parameters
!!tskip, tlength
!   0.0 2000.0
!!dc_remove?
!  .true.        
!!Length of taper at beginning and end of time series, before adding zeros
!! to make the number of points in the record a power of two.
! 0.0 0.0
!!signnpw2(<0, backup for npw2, no zpad):
! +1.0
!!smoothing: itype, ipow, df_smooth (0 = FFT df), smooth_param
!! (see above for the meaning of these input parameters):
!   2 1 0.0 0.50
!!FREQUENCY SPECIFICATION
!   0.0 50.0
!!character string to append to name of first in pair of files for output:
! .coher_linfreq_triangle_0.50hz.ipow1.col
!!Files to process (in pairs):
! file1.smc
! file2.smc
!pp: new set of parameters
!!tskip, tlength
!   0.0 2000.0
!!dc_remove?
!  .true.        
!!Length of taper at beginning and end of time series, before adding zeros
!! to make the number of points in the record a power of two.
! 0.0 0.0
!!signnpw2(<0, backup for npw2, no zpad):
! +1.0
!!smoothing: itype, ipow, df_smooth (0 = FFT df), smooth_param
!! (see above for the meaning of these input parameters):
!   2 2 0.0 0.25
!!FREQUENCY SPECIFICATION
!   0.0 50.0
!!character string to append to name of first in pair of files for output:
! .coher_linfreq_triangle_0.25hz.ipow2.col
!!Files to process (in pairs):
! file1.smc
! file2.smc
! stop

! Dates: 01/31/05 - Written by D.M. Boore, patterned after SMC2FS2
!        10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!        09/07/18 - Update control file, added better smoothing options and frequency specification (following smc2fs2)
!
! 
!
! Dimension and declaration statements:
!
      complex x(:), y(:), xy(:), xy_smoothed(:)
      allocatable :: x, y, xy, xy_smoothed
      
      integer ilow(:), icntr(:), ihigh(:)
      allocatable :: ilow, icntr, ihigh
      real coher(:), fcntr(:), ts(:,:), 
     :     xx(:), xx_smoothed(:), 
     :     yy(:), yy_smoothed(:),
     :     xy_real(:), xy_real_smoothed(:), 
     :     xy_imag(:), xy_imag_smoothed(:), 
     :     freq_fft(:), freq_out(:)
      allocatable :: coher, fcntr, ts, 
     :     xx, xx_smoothed,  
     :     yy, yy_smoothed,  
     :     xy_real, xy_real_smoothed, 
     :     xy_imag, xy_imag_smoothed, 
     :     freq_fft, freq_out

      integer npts(2), npts_smc(2), npw2(2), nc_f_smc(2)
      integer nzpad(2)
      real sps(2)
      real tskip, tlength_in, tlength, 
     :     frctnfront, frctnback,taper_front, taper_back
      real t_smc(2), t_fft

      character char_head(11)*80, comments(500)*80, cmnts2skip(500)*80,
     : specify_frequencies*10
     
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_smc(2)*200, f_out*200,
     :        f_fs2*200, f_sum*200, path*200, c2*2, c2up*2,
     :        ext_c*10, string*200, string4name*200

      character f_ctl*60
      logical f_ctl_exist, f_smc_exist
      logical smc_format
      logical dc_remove, detrend
      logical freq_out_allocated
      
      character date_ctl_correct*8, date_ctl_in*30

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMCCOHER.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') f_ctl = 'smccoher.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '09/07/18'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; '//
     :       'the proper date is '//
     :        date_ctl_correct(1:nc_date_ctl_correct)
        write(*,'(a)') 'Update your control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMCCOHER:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)

1000  continue
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf == 0 .or. buf(1:1) == '!') go to 1000
      if(buf_upper(1:4) == 'STOP') then
        close(nu_ctl)
        close(nu_sum)
        close(nu_out)
        stop
      end if
       
      if(buf_upper(1:3) .ne. 'PP:') then   
           write(*,'(a)') ' ERROR: buf_upper(1:3) .ne. PP'//
     :       ' QUITTING'
           write(*,'(a)') 
     :     ' buf_upper(1:3) = '//buf_upper(1:3)
          close(nu_sum)
          close(nu_ctl)
          stop
      end if
      
! A NEW SET OF PROCESSING PARAMETERS
          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) tskip, tlength_in
          write(nu_sum,'(a)') 
     :     ' tskip, tlength_in ='
          write(nu_sum,*) 
     :       tskip, tlength_in
     
          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) dc_remove
          write(nu_sum,'(a,1x,l1)')  ' dc_remove = ', dc_remove

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) taper_front, taper_back
          write(nu_sum,'(a)') 
     :     ' taper_front, taper_back ='
          write(nu_sum,*) 
     :       taper_front, taper_back

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) signnpw2
          write(nu_sum,'(a)') 
     :     ' signnpw2 ='
          write(nu_sum,*) 
     :       signnpw2

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) itype, ipow, df_in, smooth_param
          write(nu_sum,'(a)') 
     :     ' itype, ipow, df_in, smooth_param ='
          write(nu_sum,*) 
     :     itype, ipow, df_in, smooth_param

          CALL skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          READ (nu_ctl,*) 
     :          f_low, f_high
          WRITE (nu_sum,'(a)') 
     :    ' f_low, f_high  = '
          WRITE (nu_sum, '(1x, 2(1x,es10.3))') 
     :      f_low, f_high
     
          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          string4name = ' '
          read(nu_ctl,'(a)') string4name
          call trim_c(string4name, nc_string4name)
          write(nu_sum,'(a)') 
     :   ' string4name = '//string4name(1:nc_string4name)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
        
      write(nu_sum,*)
      write(nu_sum,'(a)') ' Starting on another batch of time series'
      
5000  continue  ! READ ANOTHER PAIR OF TIME SERIES:
 
      write(nu_sum,*)
      write(nu_sum,'(a)') '   Starting on another pair of time series'
      
      do i = 1, 2  ! read a pair of file names

        buf = ' '
        read(nu_ctl,'(a)',end = 9999) buf
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf == 0 .or. buf(1:1) == '!') go to 1000
        if(buf_upper(1:4) == 'STOP') then
          close(nu_ctl)
          close(nu_sum)
          close(nu_out)
          stop
        end if
       
        if(buf_upper(1:3) == 'PP:') then 
      
! FINISHED WITH THE LIST OF FILES; GO BACK FOR ANOTHER BATCH

          rewind(nu_ctl)
          go to 1000
        
        end if
 
        f_smc(i) = ' '
        f_smc(i) = buf(1:nc_buf)
        call trim_c(f_smc(i), nc_f_smc(i))
        inquire(file=f_smc(i)(1:nc_f_smc(i)), exist=f_smc_exist)
        if (.not. f_smc_exist) then
          write(*,'(a)') ' ****** FILE '//f_smc(i)(1:nc_f_smc(i))// 
     :          ' DOES NOT EXIST; QUITTING ******* '
          close(nu_sum)
          close(nu_ctl)
          stop
        end if
      end do

! PASSED TESTS; PROCEED

      do i = 1, 2  ! Have to separate this do loop because need npts
                   ! for allocation statement
      
        write(*,*)
        write(*,'(1x,a)') 
     :   'Getting npts for file:', f_smc(i)(1:nc_f_smc(i))
        write(nu_sum,*)
        write(nu_sum,'(1x,a)') 
     :   'Getting npts for file:', f_smc(i)(1:nc_f_smc(i))

! GET NUMBER OF POINTS ON THIS LOOP:

        call smc_npts(f_smc(i)(1:nc_f_smc(i)), npts(i), sps(i))

        write(nu_sum,*)
        write(nu_sum, '(a, 2(1x,i5))') ' i, npts = ', i, npts(i)

        signnpw2 = +1.0
        call get_npw2(npts(i),signnpw2,npw2(i)) 
        
      end do

      npw2_max = max0(npw2(1), npw2(2))
      n4ts_alloc = npw2_max
      allocate( ts(n4ts_alloc,2))

      do i = 1, 2
! NOW READ THE DATA FROM THE FILE:

        tlength = tlength_in
        call SMCRead(f_smc(i)(1:nc_f_smc(i)), tskip, tlength,
     :               ts(1,i),  
     :               char_head, int_head, real_head, comments)
        sps(i) = real_head(2)       
        npts_smc(i) = int_head(17)  ! reset because might read in a subset
                                    ! of complete time series
        dt = 1.0/sps(i)
        fnyq = 0.5*sps(i)

!        t_smc = float(npts_smc(i)-1)*dt  !This is wrong; it perpetuates a mistake I made in grad school. 
                                          !I forget that the each data point has an implied space of 
                                          !length dt after it, so for example, A digitized time series of 2 points
                                          !has a length of 2*dt (imagine adding a third point, the length of 
                                          !the time series corresponding to the first two points is 2*dt).
        t_smc(i) = real(npts_smc(i))*dt   
        nacc_start = tskip * sps(i)
        nacc_stop  = nacc_start + tlength * sps(i)

        write(nu_sum,*)
        write(nu_sum,'(1x,a)') 
     :   'for file:', f_smc(i)(1:nc_f_smc(i))
        write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
     :     'signal_in: dt=', dt, 
     :     ' npts_smc=', npts_smc(i),' t_smc=', t_smc(i)
        write(nu_sum,'(3x,a,1x,f8.2)') 
     :     ' tlength after smcread = ', tlength
     
      end do  ! finished reading both time series
      
! CHECK FOR EQUALITY OF SPS:

      if (sps(1) .ne. sps(2)) then
          write(*,'(a)') ' sps not equal; QUITTING' 
          write(*,'(a)') ' sps(1), sps(2) = ' 
          write(*,*)       sps(1), sps(2)
          close(nu_ctl)
          close(nu_sum)
          close(nu_out)
          stop
      end if
        
! CHECK FOR EQUALITY OF NPW2, RESET TO MAX IF NOT EQUAL:

      if (npw2(1) .ne. npw2(2)) then
        write(*,'(a)') ' npw2 not equal; resetting to max' 
        write(*,'(a)') ' npw2(1), npw2(2) = ' 
        write(*,*)       npw2(1), npw2(2)
        npw2(1) = npw2_max
        npw2(2) = npw2_max
      end if
          
! SIGNAL CONDITIONING:

      do i = 1, 2

!
!     REMOVE DC, APPLY TAPER, PAD WITH ZEROS
!
        detrend = .false.
        call dcdt(ts(1,i), dt, npts_smc(i), 1, npts_smc(i), 
     :   dc_remove, detrend)

        frctnfront = taper_front/t_smc(i)
        frctnback =  taper_back/t_smc(i)
        call tprfrctn (frctnfront, frctnback, ts(1,i), npts_smc(i))

        if (npw2(i) > npts_smc(i)) then
          call zeropad( ts(1,i), npts_smc(i), npw2(i))
          nzpad(i) = npw2(i) - npts_smc(i)
        else
          nzpad(i) = 0
        end if
        
      end do
      
! COMPUTE THE SPECTRA:      
     
      allocate( x(npw2_max), y(npw2_max), 
     :          xy(npw2_max), xy_smoothed(npw2_max) )
      allocate( xx(npw2_max), xx_smoothed(npw2_max), 
     :          yy(npw2_max), yy_smoothed(npw2_max),
     :          xy_real(npw2_max), xy_real_smoothed(npw2_max), 
     :          xy_imag(npw2_max), xy_imag_smoothed(npw2_max),
     :          freq_fft(npw2_max), freq_out(npw2_max), 
     :          coher(npw2_max))

      call twofft(ts(1,1), ts(1,2), x, y, npw2_max)

! NOW COMPUTE CROSS SPECTRUM AND AUTOSPECTRA:

      do i = 1, npw2_max
        xy(i) = x(i)*conjg(y(i))
        xy_real(i) = real(xy(i)) ! store real, imag separately to use in call to smooth_interpolate
        xy_imag(i) = imag(xy(i))
        xx(i) = cabs(x(i))**2
        yy(i) = cabs(y(i))**2
      end do
      
      deallocate(ts, x, y)
      
! SET UP THE FREQUENCY ARRAY FOR OUTPUT:

      npw2d2 = npw2_max/2
      nfreq_fft = npw2d2
      
      df_fft = 1.0/(float(npw2_max)*dt)       
      
      DO j = 1, nfreq_fft
        freq_fft(j) = float(j-1)*df_fft
      END DO
 
      m_start = 1
      call locate(freq_fft, nfreq_fft, f_low, m_start)
 
      call locate(freq_fft, nfreq_fft, f_high, m_stop)
     
      IF (m_stop > nfreq_fft) THEN
        m_stop = nfreq_fft
      END IF
 
      nfreq_out = m_stop - m_start + 1
!DEBUG
      write(nu_sum,'(a,4(1x,i6))') 
     :       ' nfreq_fft, m_start, m_stop, nfreq_out = ', 
     :         nfreq_fft, m_start, m_stop, nfreq_out
!DEBUG

      DO j = 1, nfreq_out
        freq_out(j) = freq_fft(j + m_start - 1)
      END DO

      df_out = df_fft
        
      write(nu_sum,'(a,2(1x,es10.3))') 
     :       ' df_out, df_fft = ', 
     :         df_out, df_fft

      DO j = 1, nfreq_out
        IF(freq_out(j) .gt. fnyq) THEN
          nfreq_out = j -1
          print *, ' ****ATTENTION********* '
          print *, '   nfreq_out has been reset so that'//
     :            ' it is less than the Nyquist frequency'
          print *, ' ****ATTENTION********* '
          EXIT
        END IF
      END DO

! SMOOTH AND INTERPOLATE THE SPECTRA

      IF (df_in == 0.0) THEN
        df_smooth = df_fft
      ELSE
        df_smooth = df_in
      END IF
           
      freq_param = 0.0      
      CALL Smooth_interpolate (freq_fft, xx, nfreq_fft, 
     :                         freq_out, xx_smoothed, nfreq_out, 
     :                         itype, ipow, df_smooth, smooth_param,
     :                         freq_param, m_start, m_stop)
      CALL Smooth_interpolate (freq_fft, yy, nfreq_fft, 
     :                         freq_out, yy_smoothed, nfreq_out, 
     :                         itype, ipow, df_smooth, smooth_param,
     :                         freq_param, m_start, m_stop)
      CALL Smooth_interpolate (freq_fft, xy_real, nfreq_fft, 
     :                         freq_out, xy_real_smoothed, nfreq_out, 
     :                         itype, ipow, df_smooth, smooth_param,
     :                         freq_param, m_start, m_stop)
      CALL Smooth_interpolate (freq_fft, xy_imag, nfreq_fft, 
     :                         freq_out, xy_imag_smoothed, nfreq_out, 
     :                         itype, ipow, df_smooth, smooth_param,
     :                         freq_param, m_start, m_stop)

! COMPUTE THE COHERENCE AND PRINT:

! OPEN OUTPUT FILE:

      f_out = ' '
      f_out = f_smc(1)(1:nc_f_smc(1))//string4name(1:nc_string4name)
      call trim_c(f_out,nc_f_out)
      write(nu_sum,*)
      write(nu_sum,'(a)') ' Opening and writing output file '//
     :                    f_out(1:nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

      write(nu_out,'(2x,a, 9x,a, 2x,a)')
     :  'ifreq', 'freq', 'coher'

!LOOP OVER FREQUENCY

      write(nu_sum,'(a,1x,i6)') ' nfreq_out = ', nfreq_out
      do i = 1, nfreq_out
        xy_smoothed(i) = 
     :     cmplx(xy_real_smoothed(i), xy_imag_smoothed(i))
        coher(i) = cabs(xy_smoothed(i))/
     :             sqrt(xx_smoothed(i)*yy_smoothed(i))
        write(nu_out,'(1x,i6,1x,es12.5, 1x,f6.3)') 
     :    i, freq_out(i), coher(i)
      end do
      
      close(nu_out)
      deallocate(xx, xx_smoothed, yy, yy_smoothed, 
     :           xy, xy_smoothed, xy_real, xy_real_smoothed, 
     :           xy_imag, xy_imag_smoothed  )
      deallocate(freq_out, coher)
      
      go to 5000
      
9999  continue
      stop
 
 
      end
! ------------------------------------------------------------------- SMCCoher


      include '\forprogs\twofft.for'
      include '\forprogs\four1.for'
      include '\forprogs\tprfrctn.for'
      include '\forprogs\dcdt.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\locate.for'
      include '\forprogs\get_npw2.for'
      include '\forprogs\zeropad.for'
      include '\forprogs\get_path.for'
      include '\forprogs\smooth_interpolate.for'
      include '\forprogs\yintrf.for'


  