* ------------------ BEGIN SMCCORRL -----------------------------
      Program SMCCorrl

* Read a control file with info for 1 to 10 pairs of smc files.  Read
* the smc files and cross correlate them to produce a column file (actually, in
* CoPlot parlance, a wrapped ascii file) for time lags specified in the control
* file.

* NOTE: the sps must be the same for the two file being cross correlated!

* NOTE: Assume tlag_min is negative

* Here as a sample input file:

*Control file for SMCCORRL      ! first line
*Name of output file:
* smccorrl.asc
*1.1st series,2.2nd series,3.column tag,3.tlag_min, tlag_max
* c:\chichidt\a800\wnt1u.smc
* c:\chichidt\a900\tcu1291u.smc
* wntt129_1                      ! up to 8 characters, starting in any column
* -30.0 30.0     ! tlag_min, tlag_max (plus lag on output mean series 1 delayed)
* c:\chichidt\a800\wnt2u.smc
* c:\chichidt\a900\tcu1292u.smc
* wntt129_2                      ! up to 8 characters, starting in any column
* -30.0 30.0     ! tlag_min, tlag_max (plus lag on output mean series 1 delayed)
* c:\chichidt\a800\wnt3u.smc
* c:\chichidt\a900\tcu1293u.smc
* wntt129_3                      ! up to 8 characters, starting in any column
* -30.0 30.0     ! tlag_min, tlag_max (plus lag on output mean series 1 delayed)
*stop

* Dates: 01/18/01 - Written by D. Boore, based on SMC2ASC
*        11/03/06 - Modernize handling of character strings
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats


      parameter(nts_max = 10)
      real sps(nts_max)
      real tlag_min(nts_max), tlag_max(nts_max)
      integer jlag(nts_max)

      real ts_1(:,:), ts_2(:,:), xcorr(:,:), tlag(:,:)
      complex cwork(:)
      allocatable :: ts_1, ts_2, cwork, xcorr, tlag

      integer npts_1(nts_max), npts_2(nts_max)
      real real_head(50)
      integer int_head(48), nc_f_ts_1(nts_max), nc_f_ts_2(nts_max)
      character char_head(11)*80, comments(500)*80
      character f_ctl*200, f_out*200, buf*80, buf4*4,
     :  colhead(nts_max)*11, xcorr_c(nts_max)*11, idtag*80
      character f_ts_1(nts_max)*200, f_ts_2(nts_max)*200
      logical f_ctl_exist, f_ts_exist

* NOTE: need f_ts_1, f_ts_2 to be arrays because need to read npts in each
* for the dynamic array allocation before actually reading the data from each.

*      npts_max = 140000            ! depends on dimensions statement
      
* Get name of input file:
      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smccorrl.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smccorrl.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

* Open file 
      call get_lun(nu_ctl)      
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

* Read output file name:
*   Skip text lines:
      call skip(nu_ctl,2)
      f_out = ' '
      read(nu_ctl, '(a)') f_out
      call trim_c(f_out,nc_f_out)

* Loop over sets of time series:
*  Skip text lines:
      call skip(nu_ctl,1)

      nts = 0

1000  continue

* Read the first file name:
        buf = ' '
        read(nu_ctl, '(a)') buf
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)

* Check for "STOP":
        if (buf4 .eq. 'STOP') go to 2000

* Check for nts > max number
        if (nts + 1 .gt. nts_max) then
           write(*,*)
           write(*, '(a   )') 
     :    ' nts+1 > nts_max; ignoring remaining ts'
           write(*,*)
           go to 2000
        end if

* Passed tests; proceed
        nts = nts + 1
        f_ts_1(nts) = ' '
        f_ts_1(nts) = buf(1:nc_buf)
        nc_f_ts_1(nts) = nc_buf
        f_ts_2(nts) = ' '
        read(nu_ctl, '(a)') f_ts_2(nts)
        call upstr(f_ts_2(nts))
        call trim_c(f_ts_2(nts),nc_f_ts_2(nts))

        inquire(file=f_ts_1(nts), exist=f_ts_exist)
        if (.not. f_ts_exist) then
          write(*,'(3a)') 
     :          ' ****** FILE ',f_ts_1(nts)(1:nc_f_ts_1(nts)), 
     :          ' DOES NOT EXIST; QUITTING!!! ******* '
          close(nu_out)
          close(nu_ctl)
          stop
        end if

        inquire(file=f_ts_2(nts), exist=f_ts_exist)
        if (.not. f_ts_exist) then
          write(*,'(3a)') 
     :          ' ****** FILE ',f_ts_2(nts)(1:nc_f_ts_2(nts)), 
     :          ' DOES NOT EXIST; QUITTING!!! ******* '
          close(nu_out)
          close(nu_ctl)
          stop
        end if


* Read idtag:
        buf = ' '
        read(nu_ctl, '(a)') buf
        call RCC(1,buf,40,idtag,nc_id,ibgn,iend)
        if (nc_id .gt. 8) nc_id = 8
        write(*,*)
        write(*,'(2a)') '  idtag = ', idtag(1:nc_id)
        
* Read tlag_min, tlag_max:
        read(nu_ctl,*)  tlag_min(nts), tlag_max(nts)

        write(*,'(2a)') '  f_ts_1 = ', f_ts_1(nts)(1:nc_f_ts_1(nts))
        write(*,'(2a)') '  f_ts_2 = ', f_ts_2(nts)(1:nc_f_ts_2(nts))
        write(*,'(a)') 
     :      ' nts, tlag_min(nts), tlag_max(nts) = '
        write(*,*)
     :        nts, tlag_min(nts), tlag_max(nts)
 
* Set up the column headings:
        colhead(nts) = ' '
        colhead(nts) = 'ts:'//idtag(1:nc_id)//'........'
        write(*,'(2a)') ' colhead = ', colhead(nts)

* Get number of points on this loop:

        call smc_nsps(f_ts_1(nts)(1:nc_f_ts_1(nts)),npts_1(nts), sps_1)
        call smc_nsps(f_ts_2(nts)(1:nc_f_ts_2(nts)),npts_2(nts), sps_2)

        if (sps_1 .ne. sps_2) then
          write(*,'(a,1p2(1x,e10.3))') 
     :     '  ERROR!!! SPS not equal, Quitting.  sps_1, sps_2 = ',
     :      sps_1, sps_2
          close(unit=nu_ctl)
        end if

        sps(nts) = sps_1

        write(*,'(a,2(1x,i6))') ' nts, npts_1(nts) = ',nts,npts_1(nts)
        write(*,'(a,2(1x,i6))') ' nts, npts_2(nts) = ',nts,npts_2(nts)

        go to 1000  ! loop back for another file

2000  continue

      close(unit=nu_ctl)

* Figure out largest value of npts_1, npts_2, then find npw2,
* and allocate the arrays:
      call imnmax(npts_1,1,nts,1,imin_1,imax_1) 
      call imnmax(npts_2,1,nts,1,imin_2,imax_2) 
      imax = max0(imax_1, imax_2)
      signnpw2 = + 1.0
      call get_npw2(imax,signnpw2,npw2_max)
     
      write(*,*)
      write(*, '(a, 2(1x,i5))') ' nts,nts_max = ', nts, nts_max
      write(*, '(a, 2(1x,i5))') 
     :                ' imin_1, imax_1 = ', imin_1, imax_1
      write(*, '(a, 2(1x,i5))') 
     :                ' imin_2, imax_2 = ', imin_2, imax_2
      write(*, '(a, 2(1x,i5))') ' imax, npw2_max = ', imax, npw2_max

      write(*,'(a)') ' Before allocate'
      allocate( ts_1(npw2_max,nts_max), ts_2(npw2_max,nts_max),
     :          cwork(2*npw2_max))
      write(*,'(a)') ' After first allocate'
      allocate( xcorr(npw2_max,nts_max), tlag(npw2_max,nts_max))
      write(*,'(a)') ' After second allocate'
      
* Now read the data from the files:
     
      do i = 1, nts
        tskip = 0.0
        tlength = 100000.0
        write(*,'(a)') ' Before SMCRead'
        call SMCRead(f_ts_1(i)(1:nc_f_ts_1(nts)), tskip, tlength,
     :               ts_1(1,i),  
     :               char_head, int_head, real_head, comments)
        call SMCRead(f_ts_2(i)(1:nc_f_ts_2(nts)), tskip, tlength,
     :               ts_2(1,i),  
     :               char_head, int_head, real_head, comments)
        write(*,'(a)') ' After SMCRead'

* Find npts as a power of two and pad with zeros
        imax = max0(npts_1(i), npts_2(i))
        signnpw2 = + 1.0
        call get_npw2(imax,signnpw2,npw2)
* Note: the previous similar coding was to determine the overall max power of
* two.  For each pair of traces, however, it is not necessary to use the max
* although it probably would not hurt (and could then get away with a single
* column of lags)
        call zeropad(ts_1(1,i), npts_1(i), npw2)
        call zeropad(ts_2(1,i), npts_2(i), npw2)

* Do the cross-correlation:
        write(*,'(a)') ' Before call correl'
        call correl_dmb(ts_1(1,i), ts_2(1,i), npw2_max, cwork, sps(i))
!        call correl(ts_1(1,i), ts_2(1,i), npw2_max, cwork)
        write(*,'(a)') ' After call correl'

* DEBUG
        if (i .eq. 1) then
           call get_lun(nu_debug)
           open(nu_debug,file='debug.out',status='unknown')
           do j = 1, npw2
              write(nu_debug,*) j, ts_1(j,i), ts_2(j,i), cwork(j)
           end do
           write(nu_debug,*)
           do j = 1, jlag(i)
              write(nu_debug,*) tlag(j,i), xcorr(j,i)
           end do
        end if      
* DEBUG

* Fill output arrays:
        dt = 1.0/sps(i)
        j4max = int(sps(i)*tlag_max(i)) + 1
        j4min = int(sps(i)*tlag_min(i)) + 1 + npw2
        jlag(i) = 0
* Fill negative lags first
        do j = j4min, npw2
          jlag(i) = jlag(i) + 1
          tlag(jlag(i),i) = float(j-npw2-1)*dt
          xcorr(jlag(i),i) = real(cwork(j))
        end do
* Then fill for positive lags
        do j = 1, j4max
          jlag(i) = jlag(i) + 1
          tlag(jlag(i),i) = float(j-1)*dt
          xcorr(jlag(i),i) = real(cwork(j))
        end do
* DEBUG
        if (i .eq. 1) then
           write(nu_debug,*)
           do j = 1, jlag(i)
              write(nu_debug,*) tlag(j,i), xcorr(j,i)
           end do
           close(nu_debug)
        end if      
* DEBUG

 
      end do 

* Write output:

* Open output file:
      call get_lun(nu_out)      
      open(unit=nu_out, file=f_out(1:nc_f_out), status='unknown')

      write(nu_out, '(a,1x,i2)') 
     :     ' Number of variables = ', 2*nts
      write(nu_out, '(a,1x,i3)') 
     :     ' Variable names start on line # ', 4+2*nts+1
      write(nu_out, '(a,1x,i3)') 
     :     ' Data starts on line # ', 4+2*nts+2

      write(nu_out, '(2a)') ' File with input parameters: ', f_ctl
 
      do i = 1, nts
        write(nu_out, '(3a)') 
     :    ' Files to be correlated: ', 
     :    f_ts_1(i)(1:nc_f_ts_1(i)), f_ts_2(i)(1:nc_f_ts_2(i))
        write(nu_out, '(3x, a, 1p2(1x,e10.3))') 
     :    ' tlag_min, tlag_max = ', tlag_min(i), tlag_max(i)
      end do

      write(nu_out, '(10(1x,3x,a4,1x,a11))') 
     :    ('time', colhead(j), j = 1, nts)
      
* Find new jmax:
      call imnmax(jlag,1,nts,1,jmin,jmax) 

      do j = 1, jmax

* Account for unequal number of points:
        do i = 1, nts
          if (j .le. jlag(i)) then              ! fill output string with value
            write(xcorr_c(i), '(1pe11.4)') xcorr(j,i)
* Note: best if could use blanks, but CoPlot does not import this
* correctly as wrapped ascii.  It would import properly if I made
* a column file, but I am reluctant to strip off the informative header lines 
*          else                                  ! fill with blanks
*            xcorr_c(i) = ' '
          else                                  ! fill with very large number
            write(xcorr_c(i), '(1pe11.4)') 1.0e37
          end if
        end do
        write(nu_out, '(10(1x,f7.3,1x,a))')
     :    (tlag(j,i),xcorr_c(i), i = 1, nts)
      end do

      deallocate(ts_1, ts_2, cwork, xcorr, tlag)

      close(unit=nu_out)

      stop
      end
* ------------------ END SMCCORRL -----------------------------

      include '\forprogs\SMCRead.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\skip.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\rc_subs.for'
*      include '\forprogs\twofft.for'
*      include '\forprogs\realft.for'
*      include '\forprogs\four1.for'
      include '\forprogs\fork.for'
      include '\forprogs\zeropad.for'
      include '\forprogs\get_npw2.for'
      include '\forprogs\smc_nsps.for'
*      include '\forprogs\correl.for'

      SUBROUTINE correl_dmb(data1,data2,npw2,ans, sps)
      
* Dates: 11/05/06 - Modified from Numerical Recipe's correl
*                   because I could not get their version to
*                   work properly (the complex parts of
*                   ans were not zero)

      INTEGER n,NMAX
      REAL data1(npw2),data2(npw2)
      COMPLEX ans(npw2)
      PARAMETER (NMAX=140000)   ! Note: make this allocatable also!
CU    USES realft,twofft
      INTEGER i,no2
      COMPLEX work1(NMAX), work2(NMAX)
      
      ans   = cmplx(0.0,0.0)
      work1 = cmplx(0.0,0.0)
      work2 = cmplx(0.0,0.0)
      
c sample rate and frequency spacing:

      df_fft = sps/ float(npw2)

      npw2d2 = npw2 / 2

c Fill working complex array

      do j = 1, npw2   
        work1(j)=cmplx(data1(j), 0.0)
        work2(j)=cmplx(data2(j), 0.0)
      end do

c     FFT to get spectrum

      call fork(npw2,work1,+1.0)
      call fork(npw2,work2,+1.0)

      do j = 1, npw2d2
* Form spectrum of correlation function
        ans(j) = work1(j)*conjg(work2(j))
      end do
* Set Nyquist value to 0.0
      ans(npw2d2+1) = 0.0
* Fill in other half of array (for a real output)
      do j = 2, npw2d2
        ans(npw2-j+2) = conjg(ans(j))
      end do
      
* Go back to time domain:

      call fork(npw2,ans,-1.0)
      
* Scale output
      do j = 1, npw2
        ans(j) = sqrt(float(npw2))/sps * ans(j)
      end do
      return
      END

* ----------------------------- BEGIN FORK --------------------------
C FAST FOURIER                                  2/15/69
C                          LX
C    CX(K) = SQRT(1.0/LX)* SUM (CX(J)*EXP(2*PI*SIGNI*I*(J-1)*(K-1)/LX))
C                          J=1                        FOR K=1,2,...,LX
C
C  THE SCALING BETWEEN FFT AND EQUIVALENT CONTINUUM OUTPUTS
C  IS AS FOLLOWS.
C
C
C     GOING FROM TIME TO FREQUENCY:
C             F(W)=DT*SQRT(LX)*CX(K)
C
C                  WHERE W(K)=2.0*PI*(K-1)*DF

*                  and    DF = 1/(LX*DT)
C
C
C     GOING FROM FREQUENCY TO TIME, WHERE THE FREQUENCY
C     SPECTRUM IS GIVEN BY THE DIGITIZED CONTINUUM SPECTRUM:
C
C             F(T)=DF*SQRT(LX)*CX(K)
*
C                  WHERE T(K)=(K-1)*DT
C
C
C  THE RESULT OF THE SEQUENCE...TIME TO FREQUENCY,POSSIBLE MODIFICATIONS
C  OF THE SPECTRUM (FOR FILTERING,ETC.), BACK TO TIME...
C  REQUIRES NO SCALING.
C
C
C  THIS VERSION HAS A SLIGHT MODIFICATION TO SAVE SOME TIME...
C  IT TAKES THE FACTOR 3.1415926*SIGNI/L OUTSIDE A DO LOOP (D.BOORE 12/8
C  FOLLOWING A SUGGESTION BY HENRY SWANGER).
C

* Some brief notes on usage:

* "signi" is a real variable and should be called either with the value "+1.0"
* of "-1.0".  The particular value used depends on the conventions being used
* in the application (e.g., see Aki and Richards, 1980, Box 5.2, pp. 129--130).

* Time to frequency:
* In calling routine,
* 
*       do i = 1, lx
*         cx(i) = CMPLX(y(i), 0.0)
*       end do
*  where y(i) is the time series and lx is a power of 2
* 
*  After calling Fork with the complex array specified above, the following 
* symmetries exist:
* 
*        cx(1)        = dc value (f = 0 * df, where df = 1.0/(lx*dt))
*        cx(lx/2 + 1) = value at Nyquist (f = (lx/2+1-1)*df = 1.0/(2*dt))
*        cx(lx)       = CONJG(cx(2))
*        cx(lx-1)     = CONJG(cx(3))
*         |           =      |
*        cx(lx-i+2)   = CONJG(cx(i))
*         |           =      |
*        cx(lx/2+2)   = CONJG(cx(lx/2))
* 
* where "CONJG" is the Fortran complex conjugate intrinsic function
* 
* This symmetry MUST be preserved if modifications are made in the frequency 
* domain and another call to Fork (with a different sign for signi) is used
* to go back to the time domain.  If the symmetry is not preserved, then the
* time domain array will have nonzero imaginary components.  There is one case
* where advantage can be taken of this, and that is to find the Hilbert 
* transform and the window of a time series with only two calls to Fork (there 
* is a short note in BSSA {GET REFERENCE} discussing this trick, which amounts 
* to zeroing out the last half of the array and multiplying all but the dc and 
* Nyquist values by 2.0; in the time domain, REAL(cx(i)) and AIMAG(cx(i)) 
* contain the filtered (if a filter was applied) and Hilbert transform of the 
* filtered time series, respectively, while CABS(cx(i)) and ATAN2(AIMAG(cx(i)), 
* REAL(cx(i))) are the window and instantaneous phase of the filtered time 
* series, respectively.

* Some references:

* Farnbach, J.S. (1975). The complex envelope in seismic signal analysis, 
* BSSA 65, 951--962. 
* He states that the factor of 2 is applied for i = 2...npw2/2 (his indices 
* start at 0, I've added 1), which is different than the next reference:

* Mitra, S.K. (2001). Digital Signal Processing, McGraw-Hill, New York.
* He gives an algorithm on p. 794 (eq. 11.81), in which the factor of 2 is 
* applied from 0 frequency to just less than Nyquist.

* 
* The easiest way to ensure the proper symmetry is to zero out the
* last half of the array (as discussed above), but the following is what
* I usually use:  
* modify (filter) only half
* of the cx array:
* 
*       do i = 1, lx/2
*         cx(i) = filter(i)*cx(i)
*       end do
* 
* where "filter(i)" is a possibly complex filter function (and recall that 
* the frequency corresponding to i is f = float(i-1)*df).  After this, fill out
* the last half of the array using
*       
*       do i = lx/2+2, lx
*         cx(i) = CONJG(cx(lx+2-j))
*       end do
* 
* Note that nothing is done with the Nyquist value.  I assume (but am not sure!)
* that this value should be 0.0
* 
* Dates: xx/xx/xx - Written by Norm Brenner(?), Jon Claerbout(?)
*        12/21/00 - Replaced hardwired value of pi with pi evaluated here,
*                     and added comments regarding usage.  Also deleted
*                     dimension specification of cx(lx) and replace it with
*                     cx(*) in the type specification statement.  I also
*                     cleaned up the formatting of the subroutine.
*        08/28/01 - Added comment about variable "signi" being real, and 
*                   added "float" in equations for "sc" and "temp", although 
*                   not strictly required.
*        06/19/02 - Added some comments about computing envelopes and
*                   instantaneous frequencies
             
 